from osgeo import gdal, osr, ogr
import os
import subprocess
import glob
import math
import json
from pgeo.utils import log
from pgeo.utils import filesystem
from pgeo.error.custom_exceptions import PGeoException, errors

log = log.logger(__name__)

# example of statistics
stats_config = {
    "descriptive_stats": {
        "force": True
    },
    "histogram": {
        "buckets": 256,
        "include_out_of_range": 0,
        "force": True
    }
}


def get_nodata_value(file_path, band=1):
    ds = gdal.Open(file_path)
    return ds.GetRasterBand(band).GetNoDataValue()


def crop_by_vector_database_olod(input_file, minlat, minlon, maxlat, maxlon, wkt, srcnodata='nodata', dstnodata='nodata'):
    # [[[1371670.86187088, 5713124.13469331], [1371670.86187088, 5885247.87691568], [1548365.06134181, 5885247.87691568], [1548365.06134181, 5713124.13469331], [1371670.86187088, 5713124.13469331]]]

    output_file_gdal_translate = filesystem.create_tmp_filename('output_', '.geotiff')
    output_file_gdal_warp = filesystem.create_tmp_filename('output_', '.geotiff')
    output_file = filesystem.create_tmp_filename('output_', '.geotiff')
    args = [
        'gdal_translate',
        '-projwin',
        str(minlat),
        str(minlon),
        str(maxlat),
        str(maxlon),
        input_file,
        output_file_gdal_translate
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)

    args = [
        'gdalwarp',
        "-q",
        "-multi",
        "-of",
        "GTiff",
        "-cutline",
        wkt,
        "-srcnodata",
        str(srcnodata),
        "-dstnodata",
        str(dstnodata),
        # -crop_to_cutline is needed otherwise the layer is not cropped
        # TODO: resolve shifting problem
        # "-crop_to_cutline",
        # "-dstalpha",
        output_file_gdal_translate,
        output_file_gdal_warp
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)

    # TODO: is it useful the third opetation?
    args = [
        'gdal_translate',
        "-a_nodata",
        str(dstnodata),
        output_file_gdal_warp,
        output_file
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)


    filesystem.remove(output_file_gdal_warp)
    filesystem.remove(output_file_gdal_translate)

    if os.path.isfile(output_file):
        return output_file
    return None


def crop_by_vector_database(raster_path, db_spatial, query_extent, query_layer):
    srcnodatavalue = get_nodata_value(raster_path)
    extent = db_spatial.query(query_extent)
    log.info(extent)
    geom = json.dumps(extent)
    g = json.loads(geom)
    log.info(g)
    obj = g[0][0]
    log.info(obj)
    obj = json.loads(obj)
    # TODO: this is hardcoded because the returning bbox is different from the one used by GDAL processing
    log.info(obj["coordinates"])
    minlat = obj["coordinates"][0][0][0]
    minlon = obj["coordinates"][0][1][1]
    maxlat = obj["coordinates"][0][2][0]
    maxlon = obj["coordinates"][0][0][1]
    db_connection_string = db_spatial.get_connection_string(True);
    return _crop_by_vector_database(raster_path, query_layer, db_connection_string, minlat, minlon, maxlat, maxlon, srcnodatavalue, srcnodatavalue)


# TODO: instead of the connection string pass the geometry
def _crop_by_vector_database(input_file, query, db_connection_string, minlat, minlon, maxlat, maxlon, srcnodata='nodata', dstnodata='nodata'):
   # [[[1371670.86187088, 5713124.13469331], [1371670.86187088, 5885247.87691568], [1548365.06134181, 5885247.87691568], [1548365.06134181, 5713124.13469331], [1371670.86187088, 5713124.13469331]]]


    # From ST_Extent it's needed to get 1,4,3,2 of the values
    # BBOX(1371670.86187088
    # 5713124.13469331
    # 1548365.06134181
    # 5885247.87691568)
    #
    # bounding box to pass to the layer
    #1371670.86187088 5885247.87691568 1548365.06134181 5713124.13469331
    #
    # gdal_translate -projwin 1371670.86187088 5885247.87691568 1548365.06134181 5713124.1346933 /home/vortex/Desktop/LAYERS/earthstat/output/rice_area1.tif /home/vortex/Desktop/LAYERS/tmp/test3.tif

    # gdal_rasterize -b 1 -b 2 -b 3 -burn 255 -burn 255 -burn 255 -l VETOR_SHP D:/GDAL/VETOR_SHP.shp D:/GDAL/LS_SCALE.tif
    # print db_connection_string
    #
    # query = "SELECT geom from spatial.gaul1_3857_test where adm1_code = '1620'"

    #log.info(query)

    output_file_gdal_translate = filesystem.create_tmp_filename('output_', '.geotiff')
    output_file_gdal_warp = filesystem.create_tmp_filename('output_', '.geotiff')
    output_file = filesystem.create_tmp_filename('output_', '.geotiff')
    args = [
        'gdal_translate',
        '-projwin',
        str(minlat),
        str(minlon),
        str(maxlat),
        str(maxlon),
        input_file,
        output_file_gdal_translate
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)


    args = [
        'gdalwarp',
        "-q",
        "-multi",
        "-of",
        "GTiff",
        "-cutline",
        db_connection_string,
        "-csql",
        query,
        "-srcnodata",
        str(srcnodata),
        "-dstnodata",
        str(dstnodata),
        # -crop_to_cutline is needed otherwise the layer is not cropped
        # TODO: resolve shifting problem
        # "-crop_to_cutline",
        # "-dstalpha",
        output_file_gdal_translate,
        output_file_gdal_warp
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)

    # TODO: is it useful the third opetation?
    args = [
        'gdal_translate',
        "-a_nodata",
        str(dstnodata),
        output_file_gdal_warp,
        output_file
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)


    filesystem.remove(output_file_gdal_warp)
    filesystem.remove(output_file_gdal_translate)

    if os.path.isfile(output_file):
        return output_file
    return None


def crop_by_vector_database_ok(input_file, query=None, db_connection_string=None, srcnodata='nodata', dstnodata='nodata'):
    """
    :param input_file: file to be cropped
    :param query: query that has to be passed to the db
    :param db_connection_string: connection string to the db
    :param dstnodata: set nodata on the nodata value
    :return: the output file path that has been processed, or None if there is any problem on the processing
    """
    output_file_gdal_warp = filesystem.create_tmp_filename('output_', '.geotiff')
    output_file = filesystem.create_tmp_filename('output_', '.geotiff')
    args = [
        'gdalwarp',
        "-q",
        "-multi",
        "-of",
        "GTiff",
        "-cutline",
        db_connection_string,
        "-csql",
        query,
        "-srcnodata",
        str(srcnodata),
        "-dstnodata",
        str(dstnodata),
        # -crop_to_cutline is needed otherwise the layer is not cropped
        # TODO: resolve shifting problem
        # "-crop_to_cutline",
        #"-dstalpha",
        input_file,
        output_file_gdal_warp
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)

    args = [
        'gdal_translate',
        "-a_nodata",
        str(dstnodata),
        output_file_gdal_warp,
        output_file
    ]
    try:
        print args
        #TODO: handle subprocess Error (like that is not taken)
        proc = subprocess.call(args, stdout=subprocess.PIPE, stderr=None)
    except:
        stdout_value = proc.communicate()[0]
        raise PGeoException(stdout_value, 500)


    if os.path.isfile(output_file):
        return output_file
    return None


def get_statistics(input_file, config=stats_config):
    """
    :param input_file: file to be processed
    :param config: json config file to be passed
    :return: computed statistics
    """
    log.info("get_statistics: %s" % input_file)

    if config is None:
        config = stats_config

    stats = {}
    try:
        if os.path.isfile(input_file):
            ds = gdal.Open(input_file)
            if "descriptive_stats" in config:
                stats["stats"] = _get_descriptive_statistics(ds, config["descriptive_stats"])
            if "histogram" in config:
                stats["hist"] = _get_histogram(ds, config["histogram"])
        else:
            raise PGeoException(errors[522], 404)
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())
    return stats


def get_descriptive_statistics(input_file, config):
    """
    :param input_file: file to be processed
    :param config: json config file to be passed
    :return: return and array with the min, max, mean, sd statistics per band i.e. [{"band": 1, "max": 549.0, "mean": 2.8398871527778, "sd": 17.103028971129, "min": 0.0}]
    """
    try:
        if os.path.isfile(input_file):
            ds = gdal.Open(input_file)
            return _get_descriptive_statistics(ds, config)
        else:
            raise PGeoException(errors[522], 404)
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


def get_histogram(input_file, config):
    """
    :param input_file: file to be processed
    :type string
    :param config: json config file to be passed
    :type json
    :return: return and array with the min, max, mean, sd statistics per band i.e. [{"band": 1, "buckets": 256, "values": [43256, 357, ...], "max": 998.0, "min": 0.0}]
    """
    try:
        if os.path.isfile(input_file):
            ds = gdal.Open(input_file)
            return _get_histogram(ds, config)
        else:
            raise PGeoException(errors[522], 404)
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())



def location_values(input_files, x, y, band=None):
    """
    Get the value of a (x, y) location

    # TODO:
    1) pass a json, instead of [files] pass file and id
    2) pass as well the projection used i.e. EPSG:4326
    3) for now it's used x, y as lat lon (it's not used the projection)

    :param input_files: files to be processed
    :type array
    :param x: x value (for now it's used LatLon)
    :type float
    :param y: y value (for now it's used LatLon)
    :type float
    :param band: band default=None (not yet used)
    :return: and array with the values of the (x, y) location
    """
    values = []
    for input_file in input_files:
        values.append(_location_value(input_file, x, y, band))
    return values


def _location_value(input_file, x, y, band=None):
    """
    Get the value of a (x, y) location
    :param input_file: file to be processed
    :type string
    :param x: x value
    :type float
    :param y: y value
    :type float
    :param band: band default=None (not yet used)
    :return: the value of the (x, y) location
    """
    # TODO: check with -wgs84 values instead of -geoloc that is the reference system of the image
    #cmd = "gdallocationinfo -valonly " + input_file + " -l_srs EPSG:3857 -geoloc " + str(x) + " " + str(y)
    #cmd = "gdallocationinfo -valonly " + input_file + " -geoloc " + str(x) + " " + str(y)
    cmd = "gdallocationinfo -valonly " + input_file + " -wgs84 " + str(x) + " " + str(y)

    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    output, error = process.communicate()
    return output.strip()


def _get_descriptive_statistics(ds, config):
    # variables
    force = True if "force" not in config else bool(config["force"])

    # stats
    stats = []
    for band in range(ds.RasterCount):
        band += 1
        srcband = ds.GetRasterBand(band)
        if srcband is None:
            continue
        # TODO: check why the "force" doesn't work on GetStatistics but the ComputeStatistics works
        if force:
            s = srcband.ComputeStatistics(0)
            #s = srcband.ComputeRasterMinMax(False)
        else:
            s = srcband.GetStatistics(False, force)
        if stats is None:
            continue
        #srcband.SetStatistics(float(s[0]), float(s[1]), float(s[2]), float(s[3]))
        # if math.isnan(s[2]):
        #     s[2] = "null"
        # if math.isnan(s[2]):
        #     s[3] = "null"
        if math.isnan(s[2]):
            log.warn("polygon is empty! %s " % s)
        else:
            stats.append({"band": band, "min": s[0], "max": s[1], "mean": s[2], "sd": s[3]})


    return stats


def _get_histogram(ds, config):
    #log.info("config %s " % config)
    # variables
    # TODO boolean of config value
    force = True if "force" not in config else bool(config["force"])
    buckets = 256 if "buckets" not in config else int(config["buckets"])
    include_out_of_range = 0 if "include_out_of_range" not in config else int(config["include_out_of_range"])

    # stats
    stats = []
    for band in range(ds.RasterCount):
        band += 1
        if force:
            (min, max) = ds.GetRasterBand(band).ComputeRasterMinMax(0)
        else:
            min = ds.GetRasterBand(band).GetMinimum()
            max = ds.GetRasterBand(band).GetMaximum()

        #hist = ds.GetRasterBand(band).GetDefaultHistogram( force = 0 )
        #stats.append({"band": band, "buckets": hist[2], "min": hist[0], "max": hist[1], "values": hist[3]})
        hist = ds.GetRasterBand(band).GetHistogram(buckets=buckets, min=min, max=max, include_out_of_range=include_out_of_range)
        stats.append({"band": band, "buckets": buckets, "min": min, "max": max, "values": hist})
    return stats


def get_authority(file_path):
    """
    @param file_path: path to the file
    @type file_path: string
    @return: AuthorityName, AuthorityCode
    """
    ds = gdal.Open( file_path )
    prj = ds.GetProjection()
    srs=osr.SpatialReference(wkt=prj)
    return srs.GetAttrValue("AUTHORITY", 0),  srs.GetAttrValue("AUTHORITY", 1)


# Process HDFs
def process_hdfs(obj):
    log.info(obj)

    # extract bands
    hdfs = extract_files_and_band_names(obj["source_path"], obj["band"])

    # extract hdf bands
    single_hdfs = create_hdf_files(obj["output_path"], hdfs)

    # merge tiles
    hdf_merged = merge_hdf_files(obj["output_path"], obj["output_path"], obj["gdal_merge"])

    # do stats?
    #TODO: do stats

    # translate
    tiff = warp_hdf_file(hdf_merged, obj["output_path"], obj["output_file_name"], obj["gdalwarp"])

    #add overviews
    if obj.has_key("gdaladdo"):
        tiff = overviews_tif_file(tiff, obj["gdaladdo"]["parameters"], obj["gdaladdo"]["overviews_levels"])

    return tiff


def extract_files_and_band_names(path, band):
    bands = []
    hdfs = glob.glob(path + "/*.hdf")
    for f in hdfs:
        gtif = gdal.Open(f)
        sds = gtif.GetSubDatasets()
        bands.append(sds[int(band) - 1][0])
    return bands


def create_hdf_files(output_path, files):
    log.info("Create HDF Files")
    i = 0;
    for f in files:
        cmd = "gdal_translate '" + f + "' " + output_path + "/" + str(i) + ".hdf"
        log.info(cmd)
        process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        output, error = process.communicate()
        i += 1;
        #TODO catch the error
        log.info(output)
        log.warn(error)


def merge_hdf_files(source_path, output_path, parameters=None):
    log.info("Merge HDF Merge")
    output_file = output_path + "/output.hdf"

    # creating the cmd
    cmd = "gdal_merge.py "
    for key in parameters.keys():
        cmd += " " + key + " " + str(parameters[key])
    cmd += " " + source_path + "/*.hdf -o " + output_file

    log.info(cmd)
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    output, error = process.communicate()
    log.info(output)
    log.warn(error)
    return output_file


def warp_hdf_file(source_file, output_path, output_file_name, parameters=None ):
    log.info("Warp HDF File to Ti")
    output_file = output_path + "/" + output_file_name

    cmd = "gdalwarp "
    for key in parameters.keys():
        cmd += " " + key + " " + str(parameters[key])
    cmd += " " + source_file + " " + output_file

    log.info(cmd)
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    output, error = process.communicate()
    log.info(output)
    log.warn(error)
    return output_file


def overviews_tif_file(output_file, parameters=None, overviews_levels=None):
    log.info("Create overviews")

    cmd = "gdaladdo "
    for key in parameters.keys():
        cmd += " " + key + " " + str(parameters[key])
    cmd += " " + output_file
    cmd += " " + overviews_levels

    log.info(cmd)
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    output, error = process.communicate()
    log.info(output)
    log.warn(error)
    return output_file


#! /usr/bin/python
# -*- coding: utf-8 -*-
#******************************************************************************
#
#  Purpose:  Quick Extension of gdal_calc.py. Command line raster calculator for gdal supported files.
#  Support multiple files instead of a fixed number like in gdal_calc.py
#  Author:   Simone Murzilli, simone.murzilli@gmail.com
#  Based on gdal_calc.py. Author:   Chris Yesson, chris.yesson@ioz.ac.uk
#
#******************************************************************************
#  Copyright (c) 2010, Chris Yesson <chris.yesson@ioz.ac.uk>
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#******************************************************************************

################################################################
# Command line raster calculator for gdal supported files. Use any basic arithmetic supported by numpy arrays such as +-*\ along with logical operators such as >.  Note that all files must be the same dimensions, but no projection checking is performed.  Use gdal_calc.py --help for list of options.

# example 1 - add two files together
# gdal_calculations.py --aa input1.tif --bb input2.tif --outfile=result.tif --calc="aa+bb" --alphalist=['aa','bb']

# example 2 - average of two layers
# gdal_calculations.py --aa input1.tif --bb input2.tif --outfile=result.tif --calc="(aa+bb)/2" --alphalist=['aa','bb']

# example 3 - set values of zero and below to null
# gdal_calculations.py --aa input1.tif --outfile=result.tif --calc="aa*(aa>0)" --NoDataValue=0 --alphalist=['aa']
################################################################

try:
    from osgeo import gdal
    from osgeo.gdalnumeric import *
except ImportError:
    import gdal
    from gdalnumeric import *

from optparse import OptionParser
import os
import sys

# set up some default nodatavalues for each datatype
DefaultNDVLookup={'Byte':255, 'UInt16':65535, 'Int16':-32767, 'UInt32':4294967293, 'Int32':-2147483647, 'Float32':1.175494351E-38, 'Float64':1.7976931348623158E+308}

#3.402823466E+38

################################################################
def doit(opts, AlphaList, len_argv):

    if opts.debug:
        print("gdal_calc.py starting calculation %s" %(opts.calc))

    ################################################################
    # fetch details of input layers
    ################################################################

    # set up some lists to store data for each band
    myFiles=[]
    myBands=[]
    myAlphaList=[]
    myDataType=[]
    myDataTypeNum=[]
    myNDV=[]
    DimensionsCheck=None

    # loop through input files - checking dimensions
    for i,myI in enumerate(AlphaList[0:len_argv-1]):
        myF = eval("opts.%s" %(myI))
        myBand = eval("opts.%s_band" %(myI))

        if myF:
            myFiles.append(gdal.Open(myF, gdal.GA_ReadOnly))
            # check if we have asked for a specific band...
            if myBand:
                myBands.append(myBand)
            else:
                myBands.append(1)
            myAlphaList.append(myI)
            myDataType.append(gdal.GetDataTypeName(myFiles[i].GetRasterBand(myBands[i]).DataType))
            myDataTypeNum.append(myFiles[i].GetRasterBand(myBands[i]).DataType)
            myNDV.append(myFiles[i].GetRasterBand(myBands[i]).GetNoDataValue())
            # check that the dimensions of each layer are the same
            if DimensionsCheck:
                if DimensionsCheck!=[myFiles[i].RasterXSize, myFiles[i].RasterYSize]:
                    print("Error! Dimensions of file %s (%i, %i) are different from other files (%i, %i).  Cannot proceed" % \
                          (myF,myFiles[i].RasterXSize, myFiles[i].RasterYSize,DimensionsCheck[0],DimensionsCheck[1]))
                    return
            else:
                DimensionsCheck=[myFiles[i].RasterXSize, myFiles[i].RasterYSize]

            if opts.debug:
                print("file %s: %s, dimensions: %s, %s, type: %s" %(myI,myF,DimensionsCheck[0],DimensionsCheck[1],myDataType[i]))

    ################################################################
    # set up output file
    ################################################################

    # open output file exists
    if os.path.isfile(opts.outF) and not opts.overwrite:
        if opts.debug:
            print("Output file %s exists - filling in results into file" %(opts.outF))
        myOut=gdal.Open(opts.outF, gdal.GA_Update)
        if [myOut.RasterXSize,myOut.RasterYSize] != DimensionsCheck:
            print("Error! Output exists, but is the wrong size.  Use the --overwrite option to automatically overwrite the existing file")
            return
        myOutB=myOut.GetRasterBand(1)
        myOutNDV=myOutB.GetNoDataValue()
        myOutType=gdal.GetDataTypeName(myOutB.DataType)

    else:
        # remove existing file and regenerate
        if os.path.isfile(opts.outF):
            os.remove(opts.outF)
        # create a new file
        if opts.debug:
            print("Generating output file %s" %(opts.outF))

        # find data type to use
        if not opts.type:
            # use the largest type of the input files
            myOutType=gdal.GetDataTypeName(max(myDataTypeNum))
        else:
            myOutType=opts.type

        # create file
        myOutDrv = gdal.GetDriverByName(opts.format)
        myOut = myOutDrv.Create(
            opts.outF, DimensionsCheck[0], DimensionsCheck[1], 1,
            gdal.GetDataTypeByName(myOutType), opts.creation_options)

        # set output geo info based on first input layer
        myOut.SetGeoTransform(myFiles[0].GetGeoTransform())
        myOut.SetProjection(myFiles[0].GetProjection())

        myOutB=myOut.GetRasterBand(1)
        if opts.NoDataValue!=None:
            myOutNDV=opts.NoDataValue
        else:
            myOutNDV=DefaultNDVLookup[myOutType]
        myOutB.SetNoDataValue(myOutNDV)
        # write to band
        myOutB=None
        # refetch band
        myOutB=myOut.GetRasterBand(1)

    if opts.debug:
        print("output file: %s, dimensions: %s, %s, type: %s" %(opts.outF,myOut.RasterXSize,myOut.RasterYSize,gdal.GetDataTypeName(myOutB.DataType)))

    ################################################################
    # find block size to chop grids into bite-sized chunks 
    ################################################################

    # use the block size of the first layer to read efficiently
    myBlockSize=myFiles[0].GetRasterBand(myBands[0]).GetBlockSize();
    # store these numbers in variables that may change later
    nXValid = myBlockSize[0]
    nYValid = myBlockSize[1]
    # find total x and y blocks to be read
    nXBlocks = (int)((DimensionsCheck[0] + myBlockSize[0] - 1) / myBlockSize[0]);
    nYBlocks = (int)((DimensionsCheck[1] + myBlockSize[1] - 1) / myBlockSize[1]);
    myBufSize = myBlockSize[0]*myBlockSize[1]

    if opts.debug:
        print("using blocksize %s x %s" %(myBlockSize[0], myBlockSize[1]))

    # variables for displaying progress
    ProgressCt=-1
    ProgressMk=-1
    ProgressEnd=nXBlocks*nYBlocks

    ################################################################
    # start looping through blocks of data
    ################################################################

    # loop through X-lines
    for X in range(0,nXBlocks):

        # in the rare (impossible?) case that the blocks don't fit perfectly
        # change the block size of the final piece
        if X==nXBlocks-1:
            nXValid = DimensionsCheck[0] - X * myBlockSize[0]
            myBufSize = nXValid*nYValid

        # find X offset
        myX=X*myBlockSize[0]

        # reset buffer size for start of Y loop
        nYValid = myBlockSize[1]
        myBufSize = nXValid*nYValid

        # loop through Y lines
        for Y in range(0,nYBlocks):
            ProgressCt+=1
            if 10*ProgressCt/ProgressEnd%10!=ProgressMk:
                ProgressMk=10*ProgressCt/ProgressEnd%10
                from sys import version_info
                if version_info >= (3,0,0):
                    exec('print("%d.." % (10*ProgressMk), end=" ")')
                else:
                    exec('print 10*ProgressMk, "..",')

            # change the block size of the final piece
            if Y==nYBlocks-1:
                nYValid = DimensionsCheck[1] - Y * myBlockSize[1]
                myBufSize = nXValid*nYValid

            # find Y offset
            myY=Y*myBlockSize[1]

            # create empty buffer to mark where nodata occurs
            myNDVs=numpy.zeros(myBufSize)
            myNDVs.shape=(nYValid,nXValid)

            # fetch data for each input layer
            for i,Alpha in enumerate(myAlphaList):

                # populate lettered arrays with values
                myval=BandReadAsArray(myFiles[i].GetRasterBand(myBands[i]),
                                      xoff=myX, yoff=myY,
                                      win_xsize=nXValid, win_ysize=nYValid)

                # fill in nodata values
                myNDVs=1*numpy.logical_or(myNDVs==1, myval==myNDV[i])

                # create an array of values for this block
                exec("%s=myval" %Alpha)
                myval=None


            # try the calculation on the array blocks
            try:
                myResult = eval(opts.calc)
            except:
                print("evaluation of calculation %s failed" %(opts.calc))
                raise

            # propogate nodata values 
            # (set nodata cells to zero then add nodata value to these cells)
            myResult = ((1*(myNDVs==0))*myResult) + (myOutNDV*myNDVs)

            # write data block to the output file
            BandWriteArray(myOutB, myResult, xoff=myX, yoff=myY)

    #print("Finished - Results written to %s" %opts.outF)

    return

################################################################
def main():
    print "gdal_calculations " + str(sys.argv)
    index = 0
    for args in sys.argv:
        if "alphalist" in args:
            AlphaList = sys.argv[index+1]
        index +=1

    # getting the Alphalist (list of the layer's alias)
    AlphaList = eval(AlphaList)

    parser = OptionParser()
    # define options
    parser.add_option("--calc", dest="calc", help="calculation in gdalnumeric syntax using +-/* or any numpy array functions (i.e. logical_and())")
    # hack to limit the number of input file options close to required number

    for myAlpha in AlphaList[0:len(sys.argv)-1]:
        eval('parser.add_option("--%s", dest="%s", help="input gdal raster file, note you can use any letter A-Z")' %(myAlpha, myAlpha))
        eval('parser.add_option("--%s_band", dest="%s_band", default=0, type=int, help="number of raster band for file %s")' %(myAlpha, myAlpha, myAlpha))

    parser.add_option("--alphalist", dest="",  default='', help="")
    parser.add_option("--outfile", dest="outF", default='gdal_calc.tif', help="output file to generate or fill.")
    parser.add_option("--NoDataValue", dest="NoDataValue", type=float, help="set output nodatavalue (Defaults to datatype specific values)")
    parser.add_option("--type", dest="type", help="set datatype must be one of %s" % list(DefaultNDVLookup.keys()))
    parser.add_option("--format", dest="format", default="GTiff", help="GDAL format for output file (default 'GTiff')")
    parser.add_option("--creation-option", "--co", dest="creation_options", default=[], action="append",
                      help="Passes a creation option to the output format driver. Multiple"
                           "options may be listed. See format specific documentation for legal"
                           "creation options for each format.")
    parser.add_option("--overwrite", dest="overwrite", action="store_true", help="overwrite output file if it already exists")
    parser.add_option("--debug", dest="debug", action="store_true", help="print debugging information")

    (opts, args) = parser.parse_args()
    if len(sys.argv) == 1:
        parser.print_help()
    elif not opts.calc:
        print("No calculation provided.  Nothing to do!")
        parser.print_help()
    else:
        doit(opts, AlphaList, len(sys.argv))

if __name__ == "__main__":
    main()


import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

def send_email(user, to, pwd, header, body, smtp="smtp.gmail.com", smtp_port=587):
    to = to
    gmail_user = user
    gmail_pwd = pwd
    smtpserver = smtplib.SMTP(smtp, smtp_port)
    smtpserver.ehlo()
    smtpserver.starttls()
    smtpserver.ehlo
    smtpserver.login(gmail_user, gmail_pwd)
    msg = MIMEMultipart('alternative')
    msg['Subject'] = header
    msg['From'] = gmail_user
    msg['To'] = to
    part = MIMEText(body, 'html')
    msg.attach(part)
    smtpserver.sendmail(gmail_user, to, msg.as_string())
    smtpserver.close()




from ftplib import FTP
from pgeo.config.settings import read_config_file_json
from pgeo.error.custom_exceptions import PGeoException
from pgeo.error.custom_exceptions import errors


conf = read_config_file_json('trmm2', 'data_providers')


def list_years():
    """
    List all the available years.
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            l = ftp.nlst()
            l.sort(reverse=True)
            out = []
            for s in l:
                try:
                    int(s)
                    out.append({'code': s, 'label': s})
                except:
                    pass
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except Exception, e:
        raise PGeoException(e.message, status_code=500)


def list_months(year):
    """
    List all the available years.
    @param year: e.g. '2010'
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(year)
            l = ftp.nlst()
            l.sort()
            out = []
            for s in l:
                out.append({'code': s, 'label': s})
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_days(year, month):
    """
    List all the available years.
    @param year: e.g. '2010'
    @param month: e.g. '02'
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(year)
            ftp.cwd(month)
            l = ftp.nlst()
            l.sort()
            out = []
            for s in l:
                out.append({'code': s, 'label': s})
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_layers(year, month, day):
    """
    List all the available layers for a given year and month.
    @param year: e.g. '2010'
    @param month: e.g. '02'
    @return: An array of code/label/extensions objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(year)
            ftp.cwd(month)
            ftp.cwd(day)
            l = ftp.nlst()
            l.sort()
            fao_layers = filter(lambda x: '.tif' in x, l)
            out = []
            for layer in fao_layers:
                if '.7.' in layer or '.7A.' in layer:
                    try:
                        code = layer
                        hour = layer[0:layer.index('.tif')].split('.')[2]
                        label = layer[0:layer.index('.tif')].split('.')[0]
                        label += ' ('
                        label += '-'.join([year, month, day])
                        label += ', ' + hour + ')'
                        out.append({'code': code, 'label': label, 'extensions': ['.tif', '.tfw']})
                    except:
                        pass
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_layers_subset(year, month, from_day, to_day):
    """
    List all the available layers for a given year and month.
    @param year: e.g. '2010'
    @param month: e.g. '02'
    @from_day: e.g. 01
    @to_day: e.g. 05
    @return: An array of code/label/extensions objects.
    """
    file_path_root = 'ftp://' + conf['source']['ftp']['base_url'] + conf['source']['ftp']['data_dir']
    days = map(lambda x: str(x) if x > 9 else '0'+str(x), range(int(from_day), 1+int(to_day)))
    out = []
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(year)
            ftp.cwd(month)
            for i in range(0, len(days)):
                if i > 0:
                    ftp.cwd('../')
                ftp.cwd(days[i])
                l = ftp.nlst()
                l.sort()
                fao_layers = filter(lambda x: '.tif' in x, l)
                for layer in fao_layers:
                    if '.7.' in layer or '.7A.' in layer:
                        code = layer
                        hour = layer[0:layer.index('.tif')].split('.')[2]
                        label = layer[0:layer.index('.tif')].split('.')[0]
                        label += ' ('
                        label += '-'.join([year, month, days[i]])
                        label += ', ' + hour + ')'
                        file_path = file_path_root + year + '/' + month + '/' + days[i] + '/' + code
                        out.append({
                            'file_name': code,
                            'file_path': file_path,
                            'label': label,
                            'size': None
                        })
                        code = code.replace('.tif', '.tfw')
                        file_path = file_path_root + year + '/' + month + '/' + days[i] + '/' + code
                        out.append({
                            'file_name': code,
                            'file_path': file_path,
                            'label': label,
                            'size': None
                        })
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


from ftplib import FTP
from threading import Thread
from threading import Lock
import Queue
import os
import time
from threading import Timer
from pgeo.utils import log
from pgeo.utils.filesystem import create_filesystem


log = log.logger('bulk_download_threads_manager.py')
progress_map = {}
exit_flags = {}


class BulkDownloadThread(Thread):

    bulk_download_object = None
    total_files = 0
    downloaded_files = 0

    def __init__(self, thread_name, queue, queue_lock, tab_id, target_folder):

        Thread.__init__(self)

        self.thread_name = thread_name
        self.queue = queue
        self.queue_lock = queue_lock
        self.tab_id = tab_id
        self.target_folder = target_folder

        progress_map[self.tab_id] = {}
        progress_map[self.tab_id]['status'] = 'WAITING'

    def run(self):

        while not exit_flags[self.tab_id]:

            self.queue_lock.acquire()

            if not self.queue.empty():

                self.bulk_download_object = self.queue.get()
                self.total_files = len(self.bulk_download_object['file_list'])
                progress_map[self.tab_id]['total_files'] = self.total_files
                progress_map[self.tab_id]['downloaded_files'] = 0
                progress_map[self.tab_id]['status'] = 'START'
                progress_map[self.tab_id]['progress'] = 0

                self.queue_lock.release()

                ftp = FTP(self.bulk_download_object['ftp_base_url'])

                try:
                    ftp.login()
                except Exception, e:
                    progress_map[self.tab_id]['status'] = 'ERROR'
                    exit_flags[self.tab_id] = 1
                    continue

                ftp.cwd(self.bulk_download_object['ftp_data_dir'])
                remote_files = ftp.nlst()

                for file_name in self.bulk_download_object['file_list']:

                    if file_name in remote_files:

                        ftp.sendcmd('TYPE i')
                        file_obj = file_name
                        local_file = os.path.join(self.target_folder, file_obj)
                        progress_map[self.tab_id]['status'] = 'ONGOING'

                        if not os.path.isfile(local_file):

                            with open(local_file, 'w') as f:

                                def callback(chunk):
                                    f.write(chunk)
                                ftp.retrbinary('RETR %s' % file_obj, callback)
                                self.downloaded_files += 1
                                progress_map[self.tab_id]['status'] = 'COMPLETE'
                                progress_map[self.tab_id]['progress'] = self.percent_done()
                                log.info(progress_map[self.tab_id]['progress'])
                                # self.update_progress_map()

                        else:
                            self.downloaded_files += 1
                            progress_map[self.tab_id]['status'] = 'COMPLETE'
                            progress_map[self.tab_id]['progress'] = self.percent_done()
                            log.info(progress_map[self.tab_id]['progress'])
                            # self.update_progress_map()

                ftp.quit()

            else:

                self.queue_lock.release()

            time.sleep(1)

    def percent_done(self):
        return float('{0:.2f}'.format(float(self.downloaded_files) / float(self.total_files) * 100))

    # def update_progress_map(self):
    #     progress_map[self.tab_id]['downloaded_files'] = self.downloaded_files
    #     progress_map[self.tab_id]['progress'] = self.percent_done()


class BulkDownloadManager(Thread):

    def __init__(self, source, filesystem_structure, bulk_download_objects, tab_id):
        Thread.__init__(self)
        self.bulk_download_objects = bulk_download_objects
        self.tab_id = tab_id
        self.source = source
        self.filesystem_structure = filesystem_structure
        self.target_folder = create_filesystem(self.source, self.filesystem_structure)

    def run(self):
        t = Timer(1, self.start_manager)
        t.start()
        return self.target_folder

    def start_manager(self):

        exit_flags[self.tab_id] = 0

        log.info('START | Bulk Download Manager')

        # thread_list = ['Alpha', 'Bravo', 'Charlie', 'Delta', 'Echo', 'Foxtrot', 'Golf', 'Hotel', 'India', 'Juliet']
        thread_list = ['Alpha']
        queue_lock = Lock()
        work_queue = Queue.Queue(len(self.bulk_download_objects))
        threads = []

        for thread_name in thread_list:
            thread = BulkDownloadThread(thread_name, work_queue, queue_lock, self.tab_id, self.target_folder)
            thread.start()
            threads.append(thread)

        queue_lock.acquire()
        for obj in self.bulk_download_objects:
            work_queue.put(obj)
        queue_lock.release()

        while not work_queue.empty():
            pass

        exit_flags[self.tab_id] = 1

        for t in threads:
            t.join()

        log.info('END   | Bulk Download Manager')


class BulkDownloadObject():

    ftp_base_url = None
    ftp_data_dir = None
    file_list = []

    def __init__(self, ftp_base_url, ftp_data_dir, file_list):
        self.ftp_base_url = ftp_base_url
        self.ftp_data_dir = ftp_data_dir
        self.file_list = file_list

    def __str__(self):
        s = ''
        s += str(self.ftp_base_url) + '\n'
        s += str(self.ftp_data_dir) + '\n'
        s += str(self.file_list)
        return s


# file_list = ['3B42.19980101.00.7.tfw',
#              '3B42.19980101.00.7.tif',
#              '3B42.19980101.03.7.tfw',
#              '3B42.19980101.03.7.tif',
#              '3B42.19980101.06.7.tfw',
#              '3B42.19980101.06.7.tif']
# one = BulkDownloadObject('trmmopen.gsfc.nasa.gov', '/trmmdata/GIS/1998/01/01', file_list)
# bulk_download_objects = [one]
# filesystem_structure = {
#     'year': '1998',
#     'month': '01',
#     'day': '01'
# }
# bdo = []
# bdo.append({
#     'ftp_base_url': 'trmmopen.gsfc.nasa.gov',
#     'ftp_data_dir': '/trmmdata/GIS/1998/01/01',
#     'file_list': ['3B42.19980101.00.7.tfw',
#                   '3B42.19980101.00.7.tif',
#                   '3B42.19980101.03.7.tfw',
#                   '3B42.19980101.03.7.tif',
#                   '3B42.19980101.06.7.tfw',
#                   '3B42.19980101.06.7.tif']})
# mgr = BulkDownloadManager('trmm2', filesystem_structure, bdo, 'tab_1')
# mgr.run()


from bson import json_util
from bson.objectid import ObjectId
from pgeo.error.custom_exceptions import PGeoException


def insert(client, database, collection, data):
    """
    Insert json data to the mongo db
    @param client client used to connect to mongo
    @param database
    @param collection
    @param data json to be inserted
    @return: id
    """
    try:
        return client[database][collection].insert(data)
    except Exception, e:
        raise PGeoException(e)


"""
DElete json data to the mongo db
@param client client used to connect to mongo
@param database
@param collection
@param data json to be inserted
@return: id
"""
def remove(client, database, collection, data):
    try:
        id =  client[database][collection].remove(data)
        return id
    except Exception, e:
        print "Delete ERROR ", e
        return None


def remove_by_id(client, database, collection, id):
    try:
        return client[database][collection].remove({'_id': ObjectId(id)})
    except Exception, e:
        raise PGeoException(e, status_code=500)


"""
Query mongo
@param client client used to connect to mongo
@param database
@param collection
@param query: default query will return the entire collection
@return: id
"""
def find(client, database, collection, query=None):
    if ( query != None ): cursor = client[database][collection].find(query)
    else: cursor = client[database][collection].find()
    return json_util.dumps(cursor)


'''

aggregation query builder

'''


"""
Query mongo query_builder_groupby rule
@param fields: fields to return in the select (i.e ['vendorcode', 'vendorname', 'varietycode'])
@param rule: aggregation rule if needed (i.e. avg, sum) TODO: make it optional
@param rule_field: filed to apply the rule TODO: make it optional
@return: json with groupby rule
"""
def query_builder_groupby(fields, rule, rule_field):
    fs = {}
    for  f in fields:
        fs[f] = '$' + f
    return { '$group' : { '_id': fs, rule_field : { '$'+ rule +'' : '$'+rule_field+'' } } }




import subprocess
import os
import random
import string
import glob
from pgeo.error.custom_exceptions import PGeoException
from pgeo.utils.filesystem import get_filename

# base script location
base_script_gdal_calculations = os.path.join(os.path.dirname(__file__), "scripts", "gdal_calculations.py")


def _random_char(char_numbers=5):
    """
    Create random char_numbers used for the id of the layers in the alphalist
    :param char_numbers: number of chars
    :type int
    :return: a random sequence of chars
    """
    return ''.join(random.choice(string.ascii_letters) for x in range(char_numbers)).lower()


def _create_cmd_filelist_from_path(files_path, alphalist):
    """
    Create cmd command for the list of files
    :param files_path: path to the files i.e. "/path/*.tif"
    :type string
    :param alphalist: array used as alphalist.
    :type array
    :return: return the cmd command part related
    """
    return _create_cmd_filelist_from_array(glob.glob(files_path), alphalist)

def _create_cmd_filelist_from_array(files, alphalist):
    """
    Create cmd command for the list of files
    :param files: List of the files i.e. ["/path/a.tif","/path/b.tif"]
    :type array
    :param alphalist: array used as alphalist.
    :type array
    :return: cmd command
    """
    cmd = ""
    for file_path in files:
        try:
            file_var = _random_char()
            alphalist.append(file_var)
            cmd += " --" + file_var + " " + file_path
        except Exception, e:
            print e
    return cmd


def _create_cmd_calc_sum(alphalist):
    """
    Create cmd "--calc" to SUM all the layers
    :param alphalist: List of files alias
    :type array
    :return: cmd command
    """
    cmd = ' --calc="('
    index=0
    for filevar in alphalist :
        cmd += filevar
        index += 1
        if index < len(alphalist):
            cmd += "+"
    cmd += ')"'
    return cmd

def _create_cmd_calc_avg(alphalist):
    """
    Create cmd "--calc" to AVG all the layers
    :param alphalist: List of files alias
    :type array
    :return: cmd command
    """
    cmd = ' --calc="('
    index=0
    for filevar in alphalist :
        cmd += filevar
        index += 1
        if index < len(alphalist):
            cmd += "+"
    cmd += ')/' + str(len(alphalist)) + '"'
    return cmd

def _create_cmd_calc_diff(alphalist):
    """
    Create cmd "--calc"
    Difference between the first layer and all the other layers
    :param alphalist: List of files alias
    :type array
    :return: cmd command
    """
    cmd = ' --calc="('
    index=0
    for filevar in alphalist :
        cmd += filevar
        index += 1
        if index < len(alphalist):
            cmd += "-"
    cmd += ')"'
    return cmd


def _create_cmd_calc_ratio(alphalist):
    """
    Create cmd "--calc"
    Ration between the first layer and the second layer
    :param alphalist: List of files alias
    :type array
    :return: cmd command
    """
    # TODO: doesn't make a float there are problems with the 0 values otherwise
    cmd = " --type Float32 --NoDataValue=0 "
    # TODO: check if the conversion can affect the perfomance
    conversion_file_to_float = "*1.0"

    cmd += ' --calc="('
    index=0
    for filevar in alphalist :
        cmd += filevar + conversion_file_to_float
        index += 1
        if index < len(alphalist):
            cmd += "/"
    cmd += ')"'
    if len(alphalist) > 2:
        raise PGeoException("Function Available between two layers")
    return cmd


def _create_cmd_outputfile(outputfile):
    """
    Create cmd "--output"
    :param outputfile: path of the output file
    :type string
    :return: cmd command
    """
    return " --outfile=" + outputfile


def _create_cmd_alphalist(alphalist):
    """
    Create cmd "--alphalist"
    :param alphalist: List of files alias
    :type array
    :return: cmd command
    """
    return ' --alphalist "' + str(alphalist) + '"'


def _process_layers(cmd):
    """
    Process the cmd command
    :param cmd:
    :return: True if the layer has bee processes
    """
    # using the base_script_gdal_calculations + cmd
    cmd = base_script_gdal_calculations + cmd
    print cmd
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    output, error = process.communicate()
    print output
    if error:
        raise PGeoException(error)
    return True


def calc_layers(files, outputfile, calc_type="avg", overwrite=True):
    """
    :param files: array of files or String with the path
    :type array or string: Array of files ["/path/a.tif","/path/b.tif"] or string with the path i.e. "/path/*.tif"
    :param outputfile: path of the output file
    :type: string
    :param calc_type: i.e. "avg", "sum", "diff", "ratio"
    :return: True if the layer has been processed
    """
    cmd = ""

    if overwrite:
        cmd += " --overwrite "

    alphalist = []
    if isinstance(files, basestring):
        cmd += _create_cmd_filelist_from_path(files, alphalist)
    elif isinstance(files, (list, tuple)):
        cmd += _create_cmd_filelist_from_array(files, alphalist)

    # output file
    cmd += _create_cmd_outputfile(outputfile)

    # calc
    if calc_type.lower() == "avg":
        cmd += _create_cmd_calc_avg(alphalist)
    elif calc_type.lower() == "sum":
        cmd += _create_cmd_calc_sum(alphalist)
    elif calc_type.lower() == "diff":
        cmd += _create_cmd_calc_diff(alphalist)
    elif calc_type.lower() == "ratio":
        cmd += _create_cmd_calc_ratio(alphalist)

    # alphalist
    cmd += _create_cmd_alphalist(alphalist)

    # process layers
    return _process_layers(cmd)





from ftplib import FTP
from pgeo.config.settings import read_config_file_json
from pgeo.error.custom_exceptions import PGeoException
from pgeo.error.custom_exceptions import errors


conf = read_config_file_json('trmm1', 'data_providers')


def list_years():
    """
    List all the available years.
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            l = ftp.nlst()
            l.sort()
            out = []
            years_buffer = []
            for s in l:
                if '.' not in s:
                    try:
                        float(s)
                        year = s[:4]
                        if year not in years_buffer:
                            years_buffer.append(year)
                    except ValueError:
                        pass
            for year in years_buffer:
                out.append({'code': year, 'label': year})
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_months(year):
    """
    List all the available months for the given year.
    @param year: e.g. '2010'
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            l = ftp.nlst()
            l.sort()
            out = []
            months_buffer = []
            for s in l:
                if '.' not in s:
                    try:
                        float(s)
                        if s.startswith(year):
                            month = s[4:]
                            if month not in months_buffer:
                                months_buffer.append(month)
                    except ValueError:
                        pass
            for month in months_buffer:
                out.append({'code': month, 'label': month})
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_layers(year, month):
    """
    List all the available layers for a given year and month.
    @param year: e.g. '2010'
    @param month: e.g. '02'
    @return: An array of code/label/extensions objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(year + month)
            l = ftp.nlst()
            l.sort()
            fao_layers = filter(lambda x: '00.7.1day.' in x, l)
            out = []
            for layer in fao_layers:
                try:
                    code = layer[:layer.index('.tfw')]
                    label = layer[layer.index('3B42RT.') + len('3B42RT.'):layer.index('.7')]
                    label = '3B42RT ' + label[0:4] + '-' + label[4:6] + '-' + label[6:8]
                    out.append({'code': code, 'label': label, 'extensions': ['.tif', '.tfw']})
                except:
                    pass
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


import httplib2
import os
import json
from copy import deepcopy
from urlparse import urlparse
from pgeo.geoserver.gsutils import url, prepare_upload_bundle
from pgeo.db.postgresql.postgis_utils import shapefile
from pgeo.utils import log
from pgeo.error.custom_exceptions import PGeoException, errors
from pgeo.utils.filesystem import get_filename, zip_files, unzip

log = log.logger(__name__)


class Geoserver():

    config = None

    # TODO: as converntion all the layers/styles should be in lowercase?

    def __init__(self, config, disable_ssl_certificate_validation=False):
        """
        Initialize and configure the GeoServer.
        The GeoServer properties are passed in the config file
        """
        self.config = config

        # use as parameters
        self.service_url = self.config['geoserver_master']
        self.username = self.config['username']
        self.password = self.config['password']

        if self.service_url.endswith("/"):
            self.service_url = self.service_url.strip("/")
        self.http = httplib2.Http(
            disable_ssl_certificate_validation=disable_ssl_certificate_validation)
        self.http.add_credentials(self.username, self.password)
        netloc = urlparse(self.service_url).netloc
        self.http.authorizations.append(
            httplib2.BasicAuthentication(
                (self.username, self.password),
                netloc,
                self.service_url,
                {},
                None,
                None,
                self.http
            ))
        self._cache = dict()
        self._version = None
        return None


    def publish_coveragestore(self, data, overwrite=False):

        # data = data["coverageStore"]
        log.info(data)
        name = data["name"]
        workspace = self.get_default_workspace() if "workspace" not in data else data["workspace"]
        path = data["path"]

        if not overwrite:
            if self.check_if_layer_exist(name, workspace):
                raise PGeoException(errors[520]+": %s" % name)

        # default geotiff headers and extension
        headers = get_headers("tiff")
        ext = "geotiff"

        # check if the layer is a tfw (a zipfile)
        #TODO: make it work with tif+tfw
        path_folder_tif, filename_tif, name_tif  = get_filename(path, True)
        file_tfw = os.path.join(path_folder_tif, name_tif + ".tfw")
        if os.path.isfile(file_tfw):
            # handle 'tfw' (worldimage)
            bundle = [path, file_tfw]
            log.info(bundle)
            archive = zip_files(name, bundle, path_folder_tif)
            log.info(archive)
            #message = open(archive, 'rb')
            message = archive
            headers = get_headers("zip")
            ext = "worldimage"
        elif isinstance(path, basestring):
            message = open(path, 'rb')
        else:
            message = path

        log.info(message)

        try:
            # Add layer to coveragestore
            cs_url = url(self.service_url, ["workspaces", workspace, "coveragestores", name, "file." + ext])
            self._publish_layer(cs_url, "PUT", message, headers, 201)

            #  Update metadata of the layer
            headers = get_headers("json")
            json_data = deepcopy(data)
            del json_data['name']
            # set has default enabled the layer
            if "enabled" not in json_data:
                json_data["enabled"] = True
            # json to send to geoserver
            update_layer = {
                "coverage" : json_data
            }
            cs_url = url(self.service_url, ["workspaces", workspace, "coveragestores", name, "coverages", name  + ".json"])
            log.info(cs_url)
            self._publish_layer(cs_url, "PUT", json.dumps(update_layer), headers, 200)

            # TODO: check why doesn't update the default style
            if 'defaultStyle' in json_data:
                if 'name' in json_data['defaultStyle']:
                    self.set_default_style(name, json_data['defaultStyle']['name'])
        except PGeoException, e:
            log.error(e.get_message())
            raise PGeoException(e.get_message(), e.get_status_code())
        except Exception, e:
            log.error(e)
            raise PGeoException(e)
        finally:
            # check if it always called
            self.reload_configuration_geoserver_slaves()
        return True


    def publish_postgis_table(self, data, overwrite=False):
        """
        :param datasource: datasource stored in geoserver
        :param name: name of the table in postgis
        :return:
        """
        #curl -v -u admin:geoserver -XPOST -H "Content-type: text/xml" -d "<featureType><name>buildings</name></featureType>"
        #http://localhost:8080/geoserver/rest/workspaces/acme/datasources/nyc/featuretypes

        #data = data["featureType"]

        name = data["name"]
        log.info(name)
        workspace = self.get_default_workspace() if "workspace" not in data else data["workspace"]
        datastore = self.get_default_datastore() if "datastore" not in data else data["datastore"]


        if not overwrite:
            log.warn("TODO: shapefile")
            #if self.check_if_layer_exist(name, workspace):
            #  raise PGeoException(errors[520]+": %s" % name)

        try:
            # TODO this can be done with just one request

            # Add layer to coveragestore
            headers = get_headers("xml")
            xml = "<featureType><name>{0}</name></featureType>".format(unicode(name).lower())
            cs_url = url(self.service_url, ["workspaces", workspace, "datastores", datastore, 'featuretypes'])
            self._publish_layer(cs_url, "POST", xml, headers)

            #  Update metadata of the layer
            headers = get_headers("json")
            json_data = deepcopy(data)
            del json_data['name']
            # set has default enabled the layer
            if "enabled" not in json_data:
                json_data["enabled"] = True
            # json to send to geoserver
            update_layer = {
                "featureType" : json_data
            }
            cs_url = url(self.service_url, ["workspaces", workspace, "datastores", datastore, "featuretypes", name + ".json"] )
            log.info(cs_url)
            self._publish_layer(cs_url, "PUT", json.dumps(update_layer), headers, 200)

            # TODO: check why doesn't update the default style
            if 'defaultStyle' in json_data:
                if 'name' in json_data['defaultStyle']:
                    log.info("change default style style")
                    self.set_default_style(name, json_data['defaultStyle']['name'])
        except PGeoException, e:
            log.error(e.get_message())
            raise PGeoException(e.get_message(), e.get_status_code())
        except Exception, e:
            log.error(e)
            raise PGeoException(e)
        finally:
            # check if it always called
            self.reload_configuration_geoserver_slaves()
        return True



    def _publish_layer(self, cs_url, c_type, message, headers, expected_code=201):
        try:
            log.info("%s %s" % (cs_url, headers))
            headers, response = self.http.request(cs_url, c_type, message, headers)
            self._cache.clear()
            log.info("%s %s %s" % (message, headers, response))

            if headers.status != expected_code:
            #raise UploadError(response)
                log.info(headers)
                log.info(response)
                raise PGeoException(response, headers.status)
        except PGeoException, e:
            log.error(e)
            raise PGeoException(e.get_message(), e.get_status_code())
        except Exception, e:
            log.error(e)
            raise PGeoException(e)
        finally:
            if hasattr(message, "close"):
                # reload geoserver cluster
                self.reload_configuration_geoserver_slaves()
                message.close()
                return True
            # if archive is not None:
            #     log.warn('call nlink(archive) : ' + archive)
            #     # nlink(archive)

    def update_layer_metadata(self, name, data, c_type="json"):
        """
        Update the layer by json or xml
        @param name: name of the layer
        @type name: string
        @param data: json
        @type name: string cointaining the data to update i.e. json '{"layer":{"title":"title of the layer"}}' or xml <layer><title>title of the layer</title></layer>
        @param type: string that can be "json" or "xml"
        @type name: string
        @return: True if updated
        """
        try:
            headers = get_headers(c_type)
            cs_url = url(self.service_url, ["layers", name + "." + c_type])
            log.info(cs_url)
            self._publish_layer(cs_url, "PUT", data, headers, 200)
        except Exception, e:
            log.error(e)
            raise PGeoException(e)
        finally:
            # check if it always called
            self.reload_configuration_geoserver_slaves()
            return True

    def delete_coveragestore(self, name, workspace=None, purge=True, recurse=True):
        if workspace is None:
            workspace = self.get_default_workspace()

        # TODO: it makes two, calls, so probably it's better just handle the delete code
        if self.check_if_layer_exist(name, workspace):
            cs_url = url(self.service_url, ["workspaces", workspace, "coveragestores", name])
            return self.delete(cs_url, purge, recurse)

    def delete(self, rest_url, purge=False, recurse=False):
        """
        send a delete request
        """

        #params aren't supported fully in httplib2 yet, so:
        params = []

        # purge deletes the SLD from disk when a style is deleted
        if purge:
            params.append("purge=true")

        # recurse deletes the resource when a layer is deleted.
        if recurse:
            params.append("recurse=true")

        if params:
            rest_url = rest_url + "?" + "&".join(params)

        headers = get_headers("xml")
        response, content = self.http.request(rest_url, "DELETE", headers=headers)
        self._cache.clear()

        if response.status == 200:
            # reload geoservers slaves configurations
            self.reload_configuration_geoserver_slaves()
            return True
        else:
           raise PGeoException(content, headers.status)

    def check_if_layer_exist(self, name, workspace=None):
        if workspace is None:
            workspace = self.get_default_workspace()
        layername = workspace +":" + name
        cs_url = url(self.service_url, ["layers", layername + ".json"])
        log.info("checking coverage exists: %s (%s)" % (name, cs_url))
        response, content = self.http.request(cs_url, "GET")
        if response.status == 200:
            return True
        elif response.status == 404:
            return False
        return False


    def set_default_style(self, name, stylename, enabled=True):
        """
        Method used to change the default style of a layer
        :param stylename: the name of the style to set ad default one
        :param layername: the name of the layer
        :param enabled: enable/disable the style
        :return:
        """
        # curl -v -u $GEOSERVER_PASSWORD -XPUT -H "Content-type: text/xml" -d "<layer><defaultStyle><name>$DEFAULT_STYLE</name></defaultStyle><enabled>$ENABLED</enabled></layer>" $GEOSERVER_URL/rest/layers/$GEOSERVER_WORKSPACE:$NAME
        headers = get_headers("xml")
        xml = "<layer><defaultStyle><name>{0}</name></defaultStyle><enabled>{1}</enabled></layer>".format(unicode(stylename).lower(),  unicode(str(enabled).upper()))
        cs_url = url(self.service_url, ["layers", name])
        log.info("Change Layer: %s  default style in %s" % (name, stylename))
        headers, response = self.http.request(cs_url, "PUT", xml, headers)
        log.info(cs_url)
        if headers.status == 200:
            # reload geoserver cluster
            self.reload_configuration_geoserver_slaves()
        else:
            raise PGeoException(response, headers.status)
        return True

    # TODO: DELETE, should not be used! use pgeo.manager.manager
    def publish_shapefile(self, input_shapefile, name, overwrite=False, workspace=None, projection=None, default_style=None, layertype=None, metadata=None):
        """
        :param input_shapefile:
        :param name:
        :param overwrite:
        :param workspace:
        :param projection:
        :param default_style:
        :param layertype:
        :param metadata:
        :return:
        """

        # TODO: check if it's a shp or zip
        '''
        if zip:
            unzip "in tmp to be removed"
        '''
        datasource = self.get_default_datasource()
        shapefile.import_shapefile(datasource, input_shapefile, name)

        # publish shapefile
        self.publish_postgis_table(datasource, name)
        return "published"




    def checkIFdatasourceExist(self, name):
        print 'TODO'

    def get_default_workspace(self):
        return self.config['default_workspace']

    def get_default_datastore(self):
        return self.config['default_datastore']

    def reload_configuration_geoserver_slaves(self, force_master_reload=False):
        geoserver_cluster = self.config["geoserver_slaves"]

        if force_master_reload is True:
            geoserver_cluster.append(self.config["geoserver_master"])

        for geoserver in geoserver_cluster:
            cs_url =  url(geoserver, ["reload?recurse=true"])
            headers, response = self.http.request(cs_url, "POST")
            log.info(headers)
            if headers.status == 200:
                return True
            else:
                return False


def get_headers(c_type):
    c_type = c_type.lower()
    headers = {
        "Accept": "application/xml"
    }
    if c_type == "geotiff":
        headers["Content-type"] = "image/tiff"
    elif c_type == "tiff":
        headers["Content-type"] = "image/tiff"
    elif c_type == "xml":
        headers["Content-type"] = "application/xml"
    elif c_type == "json":
        headers["Content-type"] = "application/json"
    elif c_type == "archive":
        headers["Content-type"] = "application/archive"
    elif c_type == "zip":
        headers["Content-type"] = "application/zip"
    else:
        raise PGeoException('Headers type "%s" not supported' % type)

    return headers


import datetime


def day_of_the_year_to_date(day, year):
    """
    Convert a day of an year to a date
    @param day: day of the year
    @type string i.e. "020" or "20"
    @param year: year of reference
    @type string or int i.e. "2014" or 2014
    @return: the date of the day/year i.e. "2012-01-20"
    """
    first_of_year = datetime.datetime(int(year), 1, 1).replace(month=1, day=1)
    ordinal = first_of_year.toordinal() - 1 + int(day)
    return datetime.date.fromordinal(ordinal)


def day_of_the_year_to_dekad(day, year):
    """
    Convert a day of an year to a date
    @param day: day of the year
    @type string i.e. "020" or "20"
    @param year: year of reference
    @type string or int i.e. "2014" or 2014
    @return: the dekad code i.e. "08-1" (month-dekad)
    """
    d = day_of_the_year_to_date(day, year)
    if d.day in range(1, 11):
        return create_dekad_code(d.month, 1)
    elif d.day in range(11, 21):
        return create_dekad_code(d.month, 2)
    elif d.day in range(21, 32):
        return create_dekad_code(d.month, 3)


def create_dekad_code(month, dekad):
    c = '0' if month < 10 else ''
    return c + str(month) + '-' + str(dekad)


def dekad_to_day_from(dekad):
    if int(dekad[3]) == 1:
        return 1
    elif int(dekad[3]) == 2:
        return 11
    elif int(dekad[3]) == 3:
        return 21

def dekad_to_day_to(dekad):
    if int(dekad[3]) == 1:
        return 10
    elif int(dekad[3]) == 2:
        return 20
    elif int(dekad[3]) == 3:
        return 31


from pgeo.config.settings import read_template
from pgeo.utils.json import dict_merge_and_convert_dates
from pgeo.metadata.db_metadata import DBMetadata
from pgeo.metadata.search import MongoSearch
from pgeo.utils import log

log = log.logger(__name__)

# REMOVE EXAMPLE
#db.layer.remove({'meContent.seCoverage.coverageSector.codes.code': {'$in': ['EARTHSTAT']}})
#db.layer.remove({'meContent.seCoverage.coverageSector.codes.code': {'$in': ['TRMM']}})


class Metadata:

    db_metadata = None
    seach = None

    def __init__(self, config):
        self.config = config
        self.db_metadata = DBMetadata(config)
        self.search = MongoSearch(config['connection'], config["database"], config['document']['layer'])
        log.info("---Metadata initialization---")
        log.info(self.db_metadata)
        log.info(self.search)



def merge_layer_metadata(template_name, data):
    """
        Merge user's data with the core metadata and the selected template
        @param template_name: Name of the template, e.g. 'modis'
        @param data: User data, in JSON format
        @return: Merged JSON
    """
    core_template = read_template('core')
    template = read_template(template_name)
    out = dict_merge_and_convert_dates(core_template, data)
    #log.info(out)
    out = dict_merge_and_convert_dates(out, template)
    #log.info(out)
    return out


import os
import json
from osgeo import ogr
from pgeo.stats.db_stats import DBStats
from pgeo.utils import log
from pgeo.gis import raster

log = log.logger(__name__)

class Stats():

    # default settings
    settings = None
    db_stats = None
    db_spatial = None

    def __init__(self, settings=None):

        self.settings = settings

        # db_stats will connect to the database
        self.db_stats = self._get_default_db("stats", True)

        # db_stats will NOT connect to the database
        self.db_spatial = self._get_default_db("spatial", True)

    def zonal_stats(self, json_stats):
        stats = None
        # TODO: a common zonalstats
        '''
        :param json_stats: json with statistics definitions
        :return: json with response
        '''

        # Raster
        # if the raster is a raster store in the datadir
        if "uid" in json_stats["raster"]:
            json_stats["raster"]["path"] = self.get_raster_path(json_stats["raster"]["uid"])

        # Vector
        # TODO: make an ENUM somewhere (i.e. database, geojson, etc)
        #log.info(json_stats["vector"]["type"])
        if json_stats["vector"]["type"] == "database":
            stats = self._zonal_stats_by_vector_database(json_stats)
        elif json_stats["vector"]["type"] == "geojson":
            log.warn("TODO: Geojson statistics")

        # Stats
        # TODO: save stats in case is needed or return statistics
        return stats

    def get_stats(self, json_stats):
        if "uid" in json_stats["raster"]:
            json_stats["raster"]["path"] = self.get_raster_path(json_stats["raster"]["uid"])
        return raster.get_statistics(json_stats["raster"]["path"])

    def get_histogram(self, json_stats):
        if "uid" in json_stats["raster"]:
            json_stats["raster"]["path"] = self.get_raster_path(json_stats["raster"]["uid"])
        return raster.get_histogram(json_stats["raster"]["path"], json_stats["stats"])

    def get_raster_path(self, uid):
        l = uid.split(":")
        return os.path.join(self.settings["folders"]["geoserver_datadir"], "data",  l[0], l[1], l[1] + ".geotiff");

    def _zonal_stats_by_vector_database(self, json_stats):
        # Stats result
        stats = []

        # raster statistics
        raster_statistics = None if "raster_stats" not in json_stats["stats"] else json_stats["stats"]["raster_stats"]

        # Raster path
        #log.info(json_stats["raster"])
        raster_path = json_stats["raster"]["path"]

        # Query Options
        vector_opt = json_stats["vector"]["options"]

        # Change SCHEMA If exists
        opt = json.dumps(vector_opt)
        # TODO: how to handle it more clearly
        # TODO: the "." (dot) should be in the schema name?
        opt = opt.replace("{{SCHEMA}}", self.db_spatial.schema)
        vector_opt = json.loads(opt)

        # retrieve query values
        select = vector_opt['query_condition']['select']
        from_query = vector_opt['query_condition']['from']
        where = None
        if "where" in vector_opt['query_condition']:
            where = vector_opt['query_condition']['where']

        # build query
        query = "SELECT " + select + " FROM "+ from_query
        if ( where is not None):
            query += " WHERE " + where

        #log.info(query)

        # parsing results
        # the column filter is used to parse the
        column_filter = vector_opt['column_filter']

        # get column filter index
        # TODO: make it dynamic
        column_filter_code_index = 0
        column_filter_label_index = 1

        srcnodatavalue = raster.get_nodata_value(raster_path)
        # log.info("SRC NODATA!: %s" % srcnodatavalue)

        # query DB
        codes = self.db_spatial.query(query)

        if codes:
            for r in codes:
                code = str(r[column_filter_code_index])
                label = str(r[column_filter_label_index])
                query_extent = "SELECT ST_AsGeoJSON(ST_Extent(geom)) FROM " + from_query + " WHERE " + column_filter + " IN (" + code + ")"
                query_layer = "SELECT * FROM " + from_query + " WHERE " + column_filter + " IN (" + code + ")"
                filepath = raster.crop_by_vector_database(raster_path, self.db_spatial, query_extent, query_layer)
                if filepath:
                    raster_stats = raster.get_statistics(filepath, raster_statistics)
                    if raster_stats:
                        obj = {"code": code, "label": label, "data": raster_stats}
                        stats.append(obj)
        return stats


    def _get_statistics(self):
        return None

    # get the default db from the settings
    def _get_default_db(self, dtype, connect=True):
        try:
            if self.settings["stats"]:
                if self.settings["db"]:
                    db_id = self.settings["stats"]["db"][dtype]
                    db = self.settings["db"][db_id]
                    if connect:
                        return DBStats(db)
                    else:
                        return db
        except:
            log.warn("No db found")
            pass






import os
from pgeo.geoserver.geoserver import Geoserver
from pgeo.metadata.metadata import Metadata
from pgeo.stats.raster import Stats
from pgeo.utils import filesystem
from pgeo.db.postgresql.postgis_utils import shapefile
from pgeo.error.custom_exceptions import PGeoException
from pgeo.utils import log
from pgeo.metadata.metadata_bridge import add_metadata_from_vector, add_metadata_from_raster, translate_from_metadata_to_geoserver


log = log.logger(__name__)


class Manager():

    metadata = None
    geoserver = None
    stats = None
    spatial_db = None

    def __init__(self, config):
        self.metadata = Metadata(config["db"]["metadata"])
        self.geoserver = Geoserver(config["geoserver"])
        self.spatial_db = config["db"]["spatial"]
        self.stats = Stats(config)


    def publish_shapefile(self, file_path, metadata_def=None, overwrite=False):
        """
        @param file_path:
        @param metadata_def:
        @param overwrite:
        @return:
        """
        try:
            # add additional layer info to the metadata i.e. bbox and EPSG code
            add_metadata_from_vector(file_path, metadata_def)
            # add additional layer info to the metadata i.e. bbox and EPSG code
            self._publish_coverage(file_path, metadata_def, translate_from_metadata_to_geoserver(metadata_def, file_path), overwrite)
        except PGeoException, e:
            raise PGeoException(e.get_message(), e.get_status_code())

    def _publish_shapefile(self, file_path, metadata_def=None, geoserver_def=None, overwrite=False):
        """
        @param file_path:
        @param layer_def:
        @param overwrite:
        @return:
        """
        try:
            # layer_def = layer_def["featureType"]

            # Unzip the shapefile
            shp_folder = filesystem.unzip(file_path)
            shp_name = filesystem.get_filename(file_path)

            # creating shp path
            shp_folder_and_name = os.path.join(shp_folder, shp_name) + ".shp"
            log.info(shp_folder_and_name)

            # sanitize the layer_name
            name = sanitize_name(shp_name)

            # getting the default workspace
            default_workspace = self.geoserver.get_default_workspace()
            if "workspace" in metadata_def["meSpatialRepresentation"]:
                default_workspace = metadata_def["meSpatialRepresentation"]["workspace"]

            # setting up the uid
            metadata_def["uid"] = default_workspace + ":" + name

            # publish shapefile on geoserver
            # TODO: merge the metadata with the default vector metadata
            if "name" not in geoserver_def:
                geoserver_def["name"] = shp_name
            if "title" not in geoserver_def:
                geoserver_def["title"] = shp_name
            if "workspace" not in geoserver_def:
                geoserver_def["workspace"] = self.geoserver.get_default_workspace()
            if "datastore" not in geoserver_def:
                geoserver_def["datastore"] = self.geoserver.get_default_datastore()

            # clean layer name
            geoserver_def["name"] = sanitize_name(geoserver_def["name"])

            # import the shapefile on postgis
            shapefile.import_shapefile(self.spatial_db, shp_folder_and_name, shp_name, True)

            # publish on metadata
            self.metadata.db_metadata.insert_metadata(metadata_def)

            # publish table on geoserver cluster
            self.geoserver.publish_postgis_table(geoserver_def, True)

            # remove files in input_shapefile
            filesystem.remove_folder(shp_folder)

        except PGeoException, e:
            log.error(e)
            self.rollback_shapefile()

    def publish_coverage(self, file_path, metadata_def=None, overwrite=False):
        """
        @param file_path:
        @param metadata_def:
        @param overwrite:
        @return:
        """
        try:
            # add additional layer info to the metadata i.e. bbox and EPSG code
            add_metadata_from_raster(file_path, metadata_def)
            # add additional layer info to the metadata i.e. bbox and EPSG code
            self._publish_coverage(file_path, metadata_def, translate_from_metadata_to_geoserver(metadata_def, file_path), overwrite)
        except PGeoException, e:
            raise PGeoException(e.get_message(), e.get_status_code())


    def _publish_coverage(self, file_path, metadata_def=None, geoserver_def=None, overwrite=False):
        """
        @param file_path:
        @param layer_def:
        @param overwrite:
        @return:
        """
        try:
            log.info(geoserver_def)
            # layer_def = layer_def["coverageStore"]

            # sanitize the layer_name
            name = sanitize_name(filesystem.get_filename(file_path))

            # getting the default workspace
            default_workspace = self.geoserver.get_default_workspace()
            if "workspace" in metadata_def["meSpatialRepresentation"]:
                default_workspace = metadata_def["meSpatialRepresentation"]["workspace"]

            # setting up the uid
            metadata_def["uid"] = default_workspace + ":" + name

            # publish coveragestore on geoserver
            # TODO: merge the metadata with the default vector metadata
            if "name" not in geoserver_def:
                geoserver_def["name"] = name
            if "title" not in geoserver_def:
                geoserver_def["title"] = name
            if "workspace" not in geoserver_def:
                geoserver_def["workspace"] = default_workspace

            # clean layer name
            #geoserver_def["name"] = sanitize_name(geoserver_def["name"])

            # publish on metadata
            self.metadata.db_metadata.insert_metadata(metadata_def)

            # publish table on geoserver cluster
            self.geoserver.publish_coveragestore(geoserver_def, True)

            # remove files and folder of the shapefile
            filesystem.remove_folder(file_path)

        except PGeoException, e:
            log.error(e)
            self.rollback_raster()


    def delete(self):
        return "TODO"

    def delete_raster(self):
        return "TODO"

    def delete_vector(self):
        return "TODO"

    def rollback_shapefile(self):
        # TODO: implement something if metadata or geoserver fails to publish or delete
        log.error("!!!!!!!!!!!!! TODO: Rollback of the imported table, metadata and geoserver layer")
        return None

    def rollback_raster(self):
        # TODO: implement something if metadata or geoserver fails to publish or delete
        log.error("!!!!!!!!!!!!! TODO: Rollback of the imported coveragestore and metadata")
        return None


def sanitize_name(name):
    """
    This method clean the name of a layer, should be avoided to use dots as names
    :param name: name of the layer
    :return: sanitized layer name
    """
    return name.replace(".", "")



class PGeoException(Exception):
    status_code = 400

    def __init__(self, message, status_code=None, payload=None):
        Exception.__init__(self, message)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        rv = dict(self.payload or ())
        rv['message'] = self.message
        rv['status_code'] = self.status_code
        return rv

    def get_message(self):
        return self.message

    def get_status_code(self):
        return self.status_code


errors = {
    510:  'Error fetching available data providers.',
    511:  'Data provider is not currently supported.',
    512:  'Source type is not currently supported.',
    513:  'Error while parsing the payload of the request.',
    # geoserver
    520: "There is already a store named",
    521: "No coverage store named",
    522:  "Layer file doesn't exists",
}


import pymongo
from pgeo.db.mongo import common
from pgeo.utils import log

log = log.logger(__name__)


class DBMetadata:

    # client = pymongo.MongoClient(settings['db']['metadata']['connection'])
    # database = settings['db']['metadata']['database']
    # document_layer = settings['db']['metadata']['document']['layer']

    def __init__(self, config):
        """
        @param config: config parameters to configure the metadata db
        @return:
        """

        self.config = config
        self.client = pymongo.MongoClient(config['connection'])
        self.database = config['database']
        self.document_layer = config['document']['layer']

    def insert_metadata(self, json):
        """
        Insert Layer Metadata in mongodb
        @param json: json data
        @return: id
        """
        return common.insert(self.client, self.database, self.document_layer, json)

    def remove_metadata(self, json):
        """
        Delete Layer Metadata in mongodb
        @param json: json data
        @return: id
        """
        return common.remove(self.client, self.database, self.document_layer, json)

    def remove_metadata_by_id(self, id):
        """
        Delete Layer Metadata in mongodb
        @param id: Metadata's id
        @return: id
        """
        return common.remove_by_id(self.client, self.database, self.document_layer, id)


from geopy.geocoders import Nominatim
from pgeo.error.custom_exceptions import PGeoException, errors
from pgeo.utils import log

geolocator = Nominatim()


def get_location(place):
    """
    @param place: place to search i.e. "FAO, roma"
    @type: string:
    @return: location obj containing the location.raw, location.address and location.latitude location.longitude
             return None if place not found
    """
    try:
        return geolocator.geocode(place)
    except Exception, e:
        raise PGeoException(e, status_code=400)


def get_reverse(lat, lon):
    """
    @param lat: place to search i.e. "52.509669"
    @type: float
    @param lon: place to search i.e. "13.376294"
    @type: float
    @return: location obj containing the location.raw, location.address and location.latitude location.longitude
             return None if place not found
    """
    try:
        return geolocator.reverse(lat, lon)
    except Exception, e:
        raise PGeoException(e, status_code=400)



import os.path
import psycopg2
import os.path
import subprocess
import osr
from urllib import urlencode
from urllib2 import urlopen
import json

from pgeo.db.postgresql.postgis_utils import util


"""
This is mostly a Python wrapper for the shp2pgsql command line utility.
"""
IMPORT_MODE_CREATE = "c"
IMPORT_MODE_APPEND = "a"
IMPORT_MODE_STRUCTURE = "p"
IMPORT_MODE_DATA = ""
IMPORT_MODE_SPATIAL_INDEX = ""


def import_shapefile(datasource, shapefile, table, overwrite=False, encoding='latin1'):
    if "schema" in datasource:
        table = datasource["schema"] + "." + table

    _import_shapefile(datasource['host'], datasource['port'], datasource['dbname'], datasource['username'], datasource['password'], shapefile, table, overwrite, encoding)
    return True

def _import_shapefile(host, port, dbname, user, password, shapefile, table, overwrite, encoding):
    conn = psycopg2.connect("host=%s  port='%s' dbname=%s user=%s password=%s" % (host, port, dbname, user, password))
    if overwrite:
        _drop_table(conn, table)

    _shape_to_postgresql(conn, shapefile, table, IMPORT_MODE_CREATE + IMPORT_MODE_DATA + IMPORT_MODE_SPATIAL_INDEX, encoding)
    _vacuum_analyze(conn, table)


def _drop_table(conn, table):
    curs = conn.cursor()
    curs.execute("DROP TABLE IF EXISTS " + table)


#TODO: how to handle the encoding?
def _shape_to_postgresql(conn, shape_path, table, mode, encoding='latin1', srid="4326", log_file=None, batch_size=1000):

    dbf_file = shape_path[0:-4] + '.dbf'
    prj_file = shape_path[0:-4] + '.prj'

    #Try detecting the SRID, by default we set to 4326 and hope the best
    srid = get_prj_srid(prj_file)

    args = [
        'shp2pgsql',  #TODO: pass the command dynamically in the config file of the server
        "-%s" % mode,
        "-W", encoding,
        "-s", srid,
        shape_path,
        table
    ]
    print args
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=log_file)

    cursor = conn.cursor()
    try:
        with p.stdout as stdout:
            for commands in util.groupsgen(util.read_until(stdout, ';'), batch_size):
                command = ''.join(commands).strip()
                if len(command) > 0:
                    cursor.execute(command)
        conn.commit()
    except:
        conn.rollback()
        raise
    finally:
        cursor.close()


# TODO: enable it to recognize Mercator used by ESRI (3857)
def get_prj_srid(prj_file, srid="4326"):
    """
    if it doesn't find the SRID uses 4326 as default one
    :param prj_file:
    :param srid:
    :return:
    """
    if os.path.isfile(prj_file):
        prj_filef = open(prj_file, 'r')
        prj_txt = prj_filef.read()
        prj_filef.close()
        srs = osr.SpatialReference()
        srs.ImportFromESRI([prj_txt])
        srs.AutoIdentifyEPSG()
        # this is used to check if the prj is 'WGS_1984_Web_Mercator_Auxiliary_Sphere' (the one used by ESRI)
        mercator_ESRI = _check_if_esri_mercator(srs.GetAttrValue('PROJCS'))
        if ( mercator_ESRI != None ):
            return mercator_ESRI
        else:
            code = srs.GetAuthorityCode(None)
            if code:
                srid = code
            else:
                #Ok, no luck, lets try with the OpenGeo service
                check_srid = _check_prj_from_webservice(prj_txt)
                if check_srid != None:
                    srid = check_srid
                '''
                query = urlencode({
                    'exact' : True,
                    'error' : True,
                    'mode' : 'wkt',
                    'terms' : prj_txt})
                print query
                webres = urlopen('http://prj2epsg.org/search.json', query)
                jres = json.loads(webres.read())
                print jres
                if jres['codes']:
                    srid = int(jres['codes'][0]['code'])  '''
    else:
        print "TODO: .prj doesn't exist try to get from the shp? or send error"
    print srid
    return srid


# TODO: this is a quick fix for ArcGIS projection error
def _check_if_esri_mercator(projcs):
    print projcs
    if projcs == 'WGS_1984_Web_Mercator_Auxiliary_Sphere':
        return 3857
    return None

def _check_prj_from_webservice(prj_txt):
    query = urlencode({
        'exact' : True,
        'error' : True,
        'mode' : 'wkt',
        'terms' : prj_txt})
    webres = urlopen('http://prj2epsg.org/search.json', query)
    jres = json.loads(webres.read())
    print jres
    if jres['codes']:
        return int(jres['codes'][0]['code']) #srid
    return None


def _vacuum_analyze(conn, table):
    isolation_level = conn.isolation_level
    conn.set_isolation_level(0)
    cursor = conn.cursor()
    try:
        cursor.execute('vacuum analyze %s;' % table)
    finally:
        cursor.close()
        conn.set_isolation_level(isolation_level)








from pgeo.utils.log import logger
from pgeo.gis.raster import get_authority
from pgeo.utils.filesystem import get_filename



log = logger("pgeo.metadata.metadata_bridge")


def translate_from_metadata_to_geoserver(metadata_json, file_path=None):
    geoserver_json = {
        #"name" : ""
        "title" : "",
        "abstract" : "",
        "enabled" : True,
        # "workspace" : "fenix",
        # "datastore" : "pgeo",
        "defaultStyle":{
        }
    }
    try:
        log.info(metadata_json)
        l = metadata_json["uid"].split(":")
        if len(l) > 1:
            geoserver_json["name"] = l[1]
            geoserver_json["workspace"] = l[0]
        else:
            log.error("there isn't a workspace associated to the uid of the layer %s" % l[0])
            geoserver_json["name"] = l[0]
    except Exception:
        pass
    try:
        geoserver_json["title"] = metadata_json["title"]["EN"]
    except Exception:
        pass
    try:
        geoserver_json["abstract"] = metadata_json["meContent"]["description"]["EN"]
    except Exception:
        pass
    try:
        print metadata_json["meSpatialRepresentation"]["seDefaultStyle"]["name"]
        geoserver_json["defaultStyle"] = {}
        geoserver_json["defaultStyle"]["name"] = metadata_json["meSpatialRepresentation"]["seDefaultStyle"]["name"]
    except Exception:
        pass

    if file_path is not None:
        geoserver_json["path"] = file_path

    return geoserver_json


# TODO: move it
def add_metadata_from_raster(file_path, metadata_json):
    try:
        autorityname, authoritycode = get_authority(file_path)
        # print autorityname
        # print authoritycode
        # TODO: add to the metadata the EPSG code
        #metadata_json["defaultStyle"]["name"] = metadata_json["meSpatialRepresentation"]["seDefaultStyle"]["name"]
    except Exception, e:
        log.error(e)
        pass
        # get boundingbox and set it

def add_metadata_from_vector(file_path, metadata_json):
    return "TODO:"

def translate_from_geoserver_to_metadata(metadata_json):
    return "TODO:"




from pgeo.geoserver.geoserver import Geoserver
from pgeo.config.settings import settings
from pgeo.utils import log
from pgeo.error.custom_exceptions import PGeoException
import sys
import random

log = log.logger("pgeo.geoserver.geoserver_test")


g = Geoserver(settings["geoserver"])

randomName = random.random()
name = "test" + str(randomName).replace(".", "")

layer_to_publish = {
    "name" : name,
    "title" : "MODIS iuhadiuh",
    "description" : "MODIS iuhadiuh",
    "workspace" : "fenix",
    "path" : "/home/vortex/Desktop/LAYERS/MODIS/AB_NDVI_4326.tif"
}


try:
    if g.publish_coveragestore(layer_to_publish, True):
        log.info("upload done")
    else:
        log.error("not uploaded")
except PGeoException, e:
    log.error(e)




from threading import Thread
from threading import Lock
import Queue
from pgeo.config.settings import read_config_file_json
import os
import uuid
import time
from threading import Timer
import urllib2
from pgeo.utils import log
from pgeo.utils.filesystem import create_filesystem
from pgeo.error.custom_exceptions import PGeoException


thread_manager_processes = {}
multi_progress_map = {}
threads_map_key = 'FENIX'
log = log.logger('download_threads_manager.py')
out_template = {
    'download_size': 0,
    'layer_name': 'unknown',
    'progress': 0,
    'total_size': 'unknown',
    'status': 'unknown',
    'thread': 'unknown',
    'key': None
}
exit_flags = {}


class LayerDownloadThread(Thread):

    file_obj = None
    file_name = None
    file_path = None
    total_size = 0
    download_size = 0

    def __init__(self, source, thread_name, queue, queue_lock, key, target_dir, tab_id, block_sz=16384):

        Thread.__init__(self)

        self.thread_name = thread_name
        self.queue = queue
        self.queue_lock = queue_lock
        self.key = key
        self.block_sz = block_sz
        self.source = source
        self.conf = read_config_file_json(self.source, 'data_providers')
        self.target_dir = target_dir
        self.tab_id = tab_id

    def run(self):

        while not exit_flags[self.tab_id]:

            self.queue_lock.acquire()

            if not self.queue.empty():

                self.file_obj = self.queue.get()
                self.file_name = self.file_obj['file_name']
                self.file_path = self.file_obj['file_path']
                self.download_size = 0

                if self.tab_id not in multi_progress_map:
                    multi_progress_map[self.tab_id] = {}
                # if self.file_name not in multi_progress_map[self.tab_id]:
                multi_progress_map[self.tab_id][self.file_name] = {}

                self.queue_lock.release()

                local_file = os.path.join(self.target_dir, self.file_name)

                if 'size' in self.file_obj:
                    self.total_size = self.file_obj['size']
                else:
                    try:
                        u = urllib2.urlopen(self.file_path)
                        meta = u.info()
                        self.total_size = int(meta.getheaders('Content-Length')[0])
                    except:
                        pass

                if self.total_size is None:
                    try:
                        u = urllib2.urlopen(self.file_path)
                        meta = u.info()
                        self.total_size = int(meta.getheaders('Content-Length')[0])
                    except:
                        pass

                # Download the file only if its size is different from the one on the FTP
                allow_layer_download = True
                try:
                    allow_layer_download = int(os.stat(local_file).st_size) < int(self.total_size)
                except OSError, e:
                    pass

                if allow_layer_download:

                    try:

                        u = urllib2.urlopen(self.file_path)
                        meta = u.info()
                        self.total_size = int(meta.getheaders('Content-Length')[0])
                        f = open(local_file, 'wb')

                        multi_progress_map[self.tab_id][self.file_name]['total_size'] = self.total_size
                        multi_progress_map[self.tab_id][self.file_name]['download_size'] = 0

                        if not os.path.isfile(local_file) or os.stat(local_file).st_size < self.total_size:
                            file_size_dl = 0
                            while self.download_size < self.total_size:
                                chunk = u.read(self.block_sz)
                                if not buffer:
                                    break
                                file_size_dl += len(chunk)
                                f.write(chunk)
                                self.download_size += len(chunk)
                                self.update_progress_map()

                        multi_progress_map[self.tab_id][self.file_name]['status'] = 'COMPLETE'
                        f.close()

                    except Exception, e:
                        log.error(str(e))
                        pass

                else:
                    multi_progress_map[self.tab_id][self.file_name]['download_size'] = self.total_size
                    multi_progress_map[self.tab_id][self.file_name]['progress'] = 100

            else:
                self.queue_lock.release()

            time.sleep(1)

    def percent_done(self):
        return float('{0:.2f}'.format(float(self.download_size) / float(self.total_size) * 100))

    def update_progress_map(self):
        multi_progress_map[self.tab_id][self.file_name]['download_size'] = self.download_size
        multi_progress_map[self.tab_id][self.file_name]['progress'] = float('{0:.2f}'.format(float(multi_progress_map[self.tab_id][self.file_name]['download_size']) / float(multi_progress_map[self.tab_id][self.file_name]['total_size']) * 100))
        multi_progress_map[self.tab_id][self.file_name]['progress'] = self.percent_done()
        multi_progress_map[self.tab_id][self.file_name]['status'] = 'DOWNLOADING'
        multi_progress_map[self.tab_id][self.file_name]['key'] = self.key


class Manager(Thread):

    def __init__(self, source, file_paths_and_sizes, filesystem_structure, tab_id):
        """
        Manager for the download threads.
        @param source: e.g. 'MODIS'
        @param file_paths_and_sizes: Array of objects with the following fields:
        ['file_name', 'size', 'label', 'file_path']
        @param filesystem_structure: e.g. {'product': 'MOD13Q1', 'year': '2014', 'day': '033'}
        """
        Thread.__init__(self)
        self.source = source
        self.file_paths_and_sizes = file_paths_and_sizes
        self.filesystem_structure = filesystem_structure
        try:
            self.target_dir = create_filesystem(self.source, self.filesystem_structure)
        except Exception, e:
            log.error(e.message)
            raise PGeoException(e.message, 500)
        self.tab_id = tab_id

    def run(self):
        t = Timer(1, self.start_manager)
        t.start()
        return self.target_dir

    def start_manager(self):

        exit_flags[self.tab_id] = 0

        log.info('START | Layers Download Manager')

        thread_list = ['Alpha', 'Bravo', 'Charlie', 'Delta', 'Echo', 'Foxtrot', 'Golf', 'Hotel', 'India', 'Juliet']
        queue_lock = Lock()
        work_queue = Queue.Queue(len(self.file_paths_and_sizes))
        threads = []

        for thread_name in thread_list:
            key = str(uuid.uuid4())
            thread = LayerDownloadThread(self.source, thread_name, work_queue, queue_lock, key, self.target_dir, self.tab_id)
            thread.start()
            if not threads_map_key in thread_manager_processes:
                thread_manager_processes[threads_map_key] = {}
            thread_manager_processes[threads_map_key][key] = thread
            threads.append(thread)

        queue_lock.acquire()
        for word in self.file_paths_and_sizes:
            work_queue.put(word)
        queue_lock.release()

        while not work_queue.empty():
            pass

        exit_flags[self.tab_id] = 1

        for t in threads:
            t.join()

        log.info('DONE | Layers Download Manager')


from pgeo.gis.gdal_calc import calc_layers



#files_path = "/home/vortex/Desktop/LAYERS/TRMM/04/*.tif"
#files_path = ["/home/vortex/Desktop/LAYERS/TRMM/Rainfall_06_2014.tif", "/home/vortex/Desktop/LAYERS/TRMM/Rainfall_05_2014.tif"]
#outputfile = "/home/vortex/Desktop/result_last_sum2.tif"

# print "Created (%s) layer %s" % (calc_layers(files_path, outputfile, "avg"), outputfile)
# print "Created (%s) layer %s" % (calc_layers(files_path, outputfile, "sum"), outputfile)
#print "Created (%s) layer %s" % (gdal_calc.calc_layers(files_path, outputfile, "sum"), outputfile)
# print "Created (%s) layer %s" % (calc_layers(files_path, outputfile, "diff"), outputfile)

# files_path = ["/home/vortex/Desktop/LAYERS/TRMM/04/*.tif"]
# outputfile = "/home/vortex/Desktop/result_last_sum.tif"
#
# print "Created (%s) layer %s" % (calc_layers(files_path, outputfile, "avg"), outputfile)


def calc_trmm(year, month):
    try:
        files_path = "/home/vortex/Desktop/LAYERS/TRMM/" + year+ "/" + month + "/*.tif"
        print files_path
        outputfile = "/home/vortex/Desktop/LAYERS/TRMM/" + year+ "/" + month + "/trmm_"+ month +"_"+ year +".tif"
        print outputfile
        calc_layers(files_path, outputfile, "sum")
    except Exception, e:
        print e
        pass

# Parameters
years = ['2012', '2013', '2014']
months = ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12']
product = '3B42RT'

for year in years:
    for month in months:
        calc_trmm(year, month)







from copy import deepcopy
from dateutil import parser


def dict_merge(a, b):
    """
    Source: https://www.xormedia.com/recursively-merge-dictionaries-in-python/
    """
    if not isinstance(b, dict):
        return b
    result = deepcopy(a)
    for k, v in b.iteritems():
        if k in result and isinstance(result[k], dict):
            result[k] = dict_merge(result[k], v)
        else:
            result[k] = deepcopy(v)
    return result


def dict_merge_and_convert_dates(a, b):
    """
    This function has been modified from the original one
    (https://www.xormedia.com/recursively-merge-dictionaries-in-python/) to convert dates
    for MongoDB insert.
    """
    if not isinstance(b, dict):
        return b
    result = deepcopy(a)
    for k, v in b.iteritems():
        if k in result and isinstance(result[k], dict):
            result[k] = dict_merge_and_convert_dates(result[k], v)
        else:
            result[k] = deepcopy(v)
            if 'version' not in k:
                try:
                    result[k] = parser.parse(result[k])
                except Exception:
                    pass
    return result


from pymongo import MongoClient
from bson.objectid import ObjectId
from pgeo.utils.date import dekad_to_day_from
from pgeo.utils.date import dekad_to_day_to
from pgeo.utils.log import logger

log = logger(__name__)


class MongoSearch():

    def __init__(self, connection, db_name, table_name):
        """

        @param connection: Connection string for MongoDB
        @param db_name: Schema name for MongoDB
        @param table_name: Document name for MongoDB
        """
        self.connection = connection
        self.db_name = db_name
        self.table_name = table_name
        self.client = MongoClient(self.connection)

    def find_layer_by_id(self, layer_id):
        """
        Find a layer's metadata by Mongo uid
        @param layer_id: MongoDB uid
        @return: Metadata document in JSON format.
        """
        return self.client[self.db_name][self.table_name].find_one({'_id': ObjectId(layer_id)})

    def find_layers_by_product(self, product=None, dekad=None, agg_type=None, confidentiality=None):
        """
        Find layers according to user's selections.
        @param product: Product ID, e.g. 'MOD13Q1'
        @param dekad: Dekad, e.g. 08-1 for August 1st dekad
        @param agg_type: Aggregation type: none, avg or da
        @return: Array of metadata document in JSON format.
        """
        q = {}
        conditions = []
        if product is not None:
            conditions.append({'meContent.seCoverage.coverageSector.codes.code': {'$in': [product]}})
        if dekad is not None:
            conditions.append({'meContent.seReferencePopulation.referencePeriod.codes.code': {'$in': [dekad]}})
        if agg_type is not None:
            conditions.append({'meStatisticalProcessing.seDatasource.seDataCompilation.aggregationProcessing': agg_type})
        if confidentiality is not None:
            conditions.append({'meAccessibility.seConfidentiality.codes.code': {'$in': [confidentiality]}})
        #q['$and'] = conditions
        q["$query"] = {'$and' : conditions}
        #q["$project"] = {'$title' : 1}
        q["$orderby"] = {'meContent.seCoverage.coverageTime.to': 1, 'title': 1, }
        log.info(q)
        return self.client[self.db_name][self.table_name].find(q)

    def find_layers_by_dekad_range(self, dekad_from, dekad_to, product=None):
        """
        Find all the layers in the specified range.
        @param dekad_from: Starting dekad of the interval
        @type dekad_from: string
        @param dekad_to: Ending dekad of the interval
        @type dekad_to: string
        @param product: Product code, e.g. 'MOD13Q1'
        @type product: string
        @return: Array of layers in the dekads range
        """
        q = []
        p = {}
        p['url'] = '$meAccessibility.seDistribution.onlineResource'
        p['year'] = {'$year': '$meContent.seCoverage.coverageTime.from'}
        p['month_from'] = {'$month': '$meContent.seCoverage.coverageTime.from'}
        p['month_to'] = {'$month': '$meContent.seCoverage.coverageTime.to'}
        p['day_from'] = {'$dayOfMonth': '$meContent.seCoverage.coverageTime.from'}
        p['day_to'] = {'$dayOfMonth': '$meContent.seCoverage.coverageTime.to'}
        if product is not None:
            p['product'] = '$meContent.seCoverage.coverageSector.codes.code'
        a = {'$project': p}
        m = {}
        m['month_from'] = {'$gte': int(dekad_from[0:2])}
        m['month_to'] = {'$lte': int(dekad_to[0:2])}
        m['day_from'] = {'$gte': dekad_to_day_from(dekad_from)}
        m['day_to'] = {'$gte': dekad_to_day_to(dekad_to)}
        if product is not None:
            m['product'] = {'$in': [product]}
        b = {'$match': m}
        q.append(a)
        q.append(b)
        return self.client[self.db_name][self.table_name].aggregate(q)['result']


    def find_all(self, field):
        """
        Find all distinct values of a specific field
        @field field: field to search
        @type field: string
         """
        q = "meContent.seCoverage.coverageSector.codes.code"

        return self.client[self.db_name][self.table_name].distinct(q)


from pgeo.stats.raster import Stats
from pgeo.config.settings import settings


layer = {
    # the stored UID in the GeoMetadata database
    "uid": "modis:test_bella_guide3",
}




#
# geostats = {
#     "code" : "226",
#     "query": "SELECT * FROM gaul0_3857 WHERE adm0_code IN ('226') ",
#     "statistics" : "all",
#     "save_stats" : True
#
# }
#

# geostats = {
#     "name" : "gaul0",
#     "query_condition" : {
#         "column_filter" : "adm0_code",
#         "select" : "distinct(adm0_code)",
#         "from"   : "gaul0_3857"
#     },
#     "save_stats" : True
# }

# geostats = {
#     "query_condition" : {
#         "column_filter" : "adm1_code",
#         "select" : "distinct(adm1_code)",
#         "from"   : "gaul1_3857",
#         "where"  : "adm0_code IN ('226')"
#     },
#     "save_stats" : True
# }
#
# geostats = {
#     "name" : "gaul1_new",
#     "query_condition" : {
#         "column_filter" : "adm1_code",
#         "select" : "adm1_code, adm1_name ",
#         "from"   : "gaul1_3857",
#         "where"  : "adm0_code IN ('150') GROUP BY adm1_code, adm1_name "
#     },
#     "save_stats" : True
# }


# REST
geostats = {
    "statlayercode" : "gaul1",
    "query_condition" : {
        "column_filter" : "adm1_code",
        "select" : "adm1_code, adm1_name ",
        "from"   : "gaul1_3857",
        "where"  : "adm0_code IN ('68') GROUP BY adm1_code, adm1_name "
    },
    "save_stats" : True
}


#New Geometadata Processing
json_stats = {
    "raster" : {
        "name" : "MODISQ13",

        # the stored UID in the GeoMetadata database (uses Geoserver Path (?) )
        "uid" : "fenix:test",

        # OR instead of the uid directly gives the path of the source layer without using the Default Path
        #"path" : "/hove/Desktop/GIS/layer.geojson",
    },

    "vector" : {

        # It's the name of the gaul spatial table
        "name" : "gaul0",

        # Database
        "type" : "database",
        "options" : {

            # used to query the db and retrieve the right codes
            "query_condition" : {
                "select": "adm1_code, adm1_name",
                "from": "{{SCHEMA}}.gaul1",
                "where": "adm0_name IN ('Somalia') GROUP BY adm1_code, adm1_name ",
            },

            # used to subquery the db to get the geometry and process the raster
            "column_filter": "adm1_code",

            # used to fill stats table (raster["name"].vector["name"])
            "stats_columns" : {
                "polygon_id" : "adm0_code",
                "label_en" : "adm0_name",
            }
        },

        # TODO: GeoJson (Problem how to save the geojson fields? Just gives back the result without saving them)
        # "type" : "geojson",
        # "path" : "/hove/Desktop/GIS/layer.geojson",
    },

    "stats" : {
        "force": True,

        # default is false (return just the json with the statistics)
        "save_stats" : True,

        # dynamically retrieved by the raster and vector names
        "table_name" : "raster.name_vector.name",

        # table.sql contains Default CREATE TABLE
        "table_definition" : "$GEOMETADATA_DEFAULT_PATH/GAUL0/table.sql",

        "table_insert": {
            # Table fiels to be used at runtime during the insertion
            # INSERT INTO table_name (polygon_id, label_en ..) VALUES (1, "Afghanistan, ..)

            "polygon_id" :  "",
            "label_en" : "",
            "fromdate" : "",
            "todate"   : "",
            "dekad" : "",
            "hist" : "",
            "max" : "",
            "min" : "",
            "sd": ""
        },

        # SQL containing INDEXES
        "table_indexes" : "$GEOMETADATA_DEFAULT_PATH/GAUL0/table_indexes.sql",

        #  default option
        "delete_tmp_files" : True
    }
}




# geostats = {
#     "name" : "gaul2",
#     "geojson" : {"type":"FeatureCollection","features":[{"type":"Feature","properties":{},"geometry":{"type":"Polygon","coordinates":[[[43.0224609375,2.3943223575350774],[43.0224609375,3.623071326235699],[44.60449218749999,3.623071326235699],[44.60449218749999,2.3943223575350774],[43.0224609375,2.3943223575350774]]]}},{"type":"Feature","properties":{},"geometry":{"type":"Polygon","coordinates":[[[46.07666015625,5.397273407690917],[46.38427734375,8.711358875426512],[50.25146484375,8.798225459016358],[46.73583984375,4.937724274302492],[46.07666015625,5.397273407690917]]]}},{"type":"Feature","properties":{},"geometry":{"type":"Polygon","coordinates":[[[42.9345703125,5.878332109674327],[44.12109374999999,8.928487062665504],[44.89013671875,7.079088026071731],[43.70361328125,4.806364708499998],[42.9345703125,5.878332109674327]]]}}]},
#     "save_stats" : False
# }

geostats = Stats(settings)
geostats.zonal_stats(json_stats)



import glob
import os
import json
from copy import deepcopy
from osgeo import gdal,osr
from pgeo.config.settings import settings
from pgeo.manager.manager import Manager, sanitize_name
from pgeo.utils.log import logger
from pgeo.utils import filesystem
from pgeo.metadata.metadata import merge_layer_metadata

log = logger("pgeo.manager.layer_utils")

# default options
raster_template = "raster"
default_metadata_json = {}

def harvest_folder(path):
    """
    Harvest files in a path
    :param path:
    :return:
    """
    manager = Manager(settings)
    types = ('*.tiff', '*.geotiff', "*.gtiff", "*.tif")
    # for each tiff, geotiff, tif, gtiff
    files_grabbed = []
    for file_type in types:
        files_grabbed.extend(glob.glob(os.path.join(path, file_type)))

    for file_path in files_grabbed:

        path, filename, name = filesystem.get_filename(file_path, True)
        #log.info("%s %s %s " % (path, filename, name))
        default_workspace = manager.geoserver.get_default_workspace()
        name = sanitize_name(name)
        if manager.geoserver.check_if_layer_exist(name, default_workspace) is False:
            metadata_file = os.path.join(path, name + ".json")
            log.info("Check Metadata File: %s" % metadata_file)
            if os.path.isfile(metadata_file):
                metadata_json = json.loads(open(metadata_file).read())
                log.info(metadata_json)
            else:
                log.warn("Metadata '%s' doesn't exists " % metadata_file)
                log.warn("Setting the default metadata file for '%s' " % name)
                # TODO: create a default metadata if the file doesn't exists it doesn't exists
                metadata = deepcopy(default_metadata_json)

            log.info("Prossessing the raster '%s' " % (name))
            metadata = merge_layer_metadata(raster_template, metadata_json)

            # process metadata with the default workspace if uid is not set in the metadata
            if "uid" not in metadata:
                metadata["uid"] = default_workspace + ":" + name
            manager.publish_coverage(file_path, metadata)

            log.info("Raster published '%s' " % (name))
        else:
            log.warn("Coverage '%s' already exists " % name)



def remove_uploded():
    return None

# Test
#harvest_folder("/home/vortex/Desktop/LAYERS/bulk_import_test/")







from pgeo.config.settings import settings
from pgeo.manager.manager import Manager
from pgeo.utils import log
import json
import copy
import random

log = log.logger("pgeo.manager.manager_test")

manager = Manager(settings)


# Publish a Shapefile
layer_def = {
        "title" : "ne_110m_dddgeography_regions3333222_poly",
        "abstract" : "ne_110m_geography_reeeeegions_polys33332",
        # "workspace" : "fenix",
        # "datastore" : "pgeo",
        "enabled" : True,
        "defaultStyle" : {
            "name" : "population"
        }
}
metadata_def = copy.deepcopy(layer_def)
manager.publish_shapefile("/home/vortex/Desktop/LAYERS/test_imports/shapefile/ne_110m_geography_regions_polys.zip", metadata_def, layer_def)



# Publish a Coveragestore
# randomName = random.random()
# name = "test" + str(randomName).replace(".", "")
# layer_coverage_def = {
#         "name" : name,
#         "title" : "aaasetta sto tile!",
#         "abstract" : "MODIS iuhadiuh",
#         "workspace" : "fenix",
#         "path" : "/home/vortex/Desktop/LAYERS/MODIS/AB_NDVI_4326.tif",
#         "defaultStyle": {
#             "name": "rain"
#         }
# }
# metadata_coverage_def = copy.deepcopy(layer_coverage_def)
# manager.publish_coverage("/home/vortex/Desktop/LAYERS/MODIS/AB_NDVI_4326.tif", metadata_coverage_def, layer_coverage_def)




import os
import uuid
import zipfile
from pgeo.config.settings import settings
from pgeo.config.settings import read_config_file_json
from pgeo.utils import log
import shutil
from pgeo.error.custom_exceptions import PGeoException
from pgeo.error.custom_exceptions import errors

log = log.logger("pgeo.utils.filesystem")

# temporary folder
folder_tmp = settings['folders']['tmp']


def create_tmp_filename(path='', extension=''):
    """
    Create the path for a tmp file and filename

    @type path: string
    @param path: path from the tmp folder
    @type extension: extension
    @param extension: i.e. .geotiff
    """
    if extension != '' and "." not in extension: extension = "." + extension
    return (os.path.join(folder_tmp, path) + str(uuid.uuid4()) + extension).encode('utf-8')

def create_tmp_folder(path=''):
    """
    Create the tmp folder from the folder_tmp

    @type path: string
    @param path: path from the tmp folder
    @return: the path to a tmp folder
    """
    return (os.path.join(folder_tmp, path) + str(uuid.uuid4())).encode('utf-8')

def create_folder_in_tmp(folder_name):
    """
    Create the tmp folder from the folder_tmp

    @type path: string
    @param path: path from the tmp folder
    @return: the path to a tmp folder
    """
    path = os.path.join(folder_tmp, folder_name)
    os.mkdir(path)
    return path

def create_tmp_file(string_value, path='', extension=''):
    """
    Create a tmp file with the passed string

    @type string_value: string
    @param string_value: string to fill the tmp file
    @type path: string
    @param path: path from the tmp folder
    @type extension: extension
    @param extension: i.e. .geotiff
    """
    filename = create_tmp_filename(path, extension)
    text_file = open(filename, "w")
    text_file.write(str(string_value))
    text_file.close()
    return filename


def unzip(filezip, path=create_tmp_folder()):
    """
    Unzip a file in the tmp folder

    @type filezip: string
    @param filezip: path to the zip file
    @type path: string
    @param path: path from the tmp folder
    @return: path to the unzipped folder
    """
    path = os.path.join(folder_tmp, path)
    log.info(path)
    with zipfile.ZipFile(filezip, "r") as z:
        z.extractall(path)
    return path


def remove(filepath):
    """
    Remove a file

    @type filepath: string
    @param filepath: path to the file
    @type path: string
    @param path: path from the tmp folder
    """
    try:
        os.remove(filepath)
    except:
        log.warn("file doesn't exists: " + str(filepath))


def remove_folder(path):
    """
    Remove a folder
    @param path: path to the folder
    @type path: string
    """
    try:
        shutil.rmtree(path, ignore_errors=True)
    except Exception, e:
        log.error(e)


def get_filename(filepath, extension=False):
    drive, path = os.path.splitdrive(filepath)
    path, filename = os.path.split(path)
    name = os.path.splitext(filename)[0]
    if extension is True:
        return path, filename, name
    else:
        return name


def zip_files(name, files, path=folder_tmp):
    extension = ".zip"
    if ".zip" in name:
        extension = ""
    zip_path = os.path.join(path, name +extension)
    log.info("Zip: '%s' from zip_files %s - %s" % (zip_path, name, files))
    zf = zipfile.ZipFile(zip_path, "w")
    zip_subdir = path
    for fpath in files:
        fdir, fname = os.path.split(fpath)

        #Add file, at correct path
        log.info(fname)
        zf.write(fpath, fname)
    zf.close()
    return zip_path


# def get_filesystem_path(source, parameters):


def create_filesystem(source, parameters):
    """
    Create the filesystem structure according to the configuration file.

    @type source: string
    @param source: e.g. 'modis'
    @type parameters: dictionary
    @param parameters: e.g. {'product': 'MOD13Q1', 'year': '2014', 'day': '033'}
    """
    conf = read_config_file_json(source, 'data_providers')['target']
    final_path = conf['target_dir']
    if not os.path.exists(conf['target_dir']):
        try:
            os.makedirs(conf['target_dir'])
        except:
            print 'Error creating: ' + str(final_path)
            os.makedirs(conf['target_dir'])
    if len(conf['folders']) > 0:
        for folder in conf['folders']:
            final_path = create_folder(conf, parameters, folder, conf['target_dir'])
    out_folder = os.path.join(final_path, 'OUTPUT')
    if not os.path.exists(out_folder):
        os.makedirs(out_folder)
    return final_path


def create_folder(conf, parameters, folder, root_folder):
    """
    Recursive function to create sub-folders.

    @type conf: dictionary
    @param conf: Data provider's configuration
    @type parameters: dictionary
    @param parameters: e.g. {'product': 'MOD13Q1', 'year': '2014', 'day': '033'}
    @type folder: dictionary
    @param folder: Dictionary describing the folder and its sub-folders, if any
    @type root_folder: string
    @param root_folder: Folders will be creating from this one
    """
    folder_name = folder['folder_name']
    if '{{' in folder_name and '}}' in folder_name:
        for key in parameters:
            if str(folder['folder_name']) == '{{' + key + '}}':
                folder_name = str(folder['folder_name']).replace('{{' + key + '}}', str(parameters[key]))
    root_folder = os.path.join(root_folder, folder_name)
    if not os.path.exists(root_folder):
        os.makedirs(root_folder)
    if 'folders' in folder and len(folder['folders']) > 0:
        for sub_folder in folder['folders']:
            root_folder = create_folder(conf, parameters, sub_folder, root_folder)
    return root_folder


def list_sources():
    try:
        path = os.path.join(os.path.dirname(os.path.dirname(__file__)) + '/' + settings['folders']['config'] + settings['folders']['data_providers'])
        out = []
        files = os.listdir(path)
        files.sort()
        for filename in files:
            if '__' not in filename:
                out.append({'code': filename, 'label': filename[:filename.index('.json')]})
        return out
    except Exception, err:
        raise PGeoException(errors[510], status_code=510)


import psycopg2
from pgeo.utils import log
from pgeo.error.custom_exceptions import PGeoException

# Logger
log = log.logger(__name__)

# TODO: use exceptions?


class DBConnection:
    con = None
    datasource = None
    autocommit = True
    schema= None


    def __init__(self, datasource):
        if DBConnection.con is None:
            try:
                log.info("---PostGIS connection initialization---")
                self.datasource = datasource
                if self.datasource["schema"]:
                    self.schema = self.datasource["schema"]
                db_connect_string = self.get_connection_string(False)
                self.con = psycopg2.connect(db_connect_string)
                log.info("Database '%s' connection opened. " % datasource['dbname'] )
            except psycopg2.DatabaseError as db_error:
                log.warn("Error :\n{0}".format(db_error))
                pass
                # raise PGeoException("Error :\n{0}".format(db_error))

    # TODO: autocommit as parameter (FOR BULK)
    def insert(self, table, insert_keys, insert_values, values):
        try:
            self.con.autocommit = self.autocommit
            cur = self.con.cursor()
            # query
            sql = "INSERT INTO " + table + " (" + insert_keys + ") VALUES (" + insert_values + ")"
            cur.execute(sql, values)
            return True
        except Exception, e:
            self.con.rollback()
            log.error("DBConnection.import_data Error: ", e)
            return False

    # TODO: autocommit as parameter (FOR BULK)
    def insert(self, table, values):
        try:
            self.con.autocommit = self.autocommit
            cur = self.con.cursor()
            # query
            sql = "INSERT INTO " + table + " VALUES (" + values + ")"
            cur.execute(sql)
            return True
        except Exception, e:
            self.con.rollback()
            log.error("DBConnection.import_data Error: ", e)
            return False

    def query(self, query):
        try:
            if self.check_query(query):
                cur = self.con.cursor()
                cur.execute(query)
                rows = cur.fetchall()
                return rows
            else:
                log.warn("Query contains invalid characters")
                raise PGeoException("Query contains invalid characters", status_code=404)
        except PGeoException, e:
                self.con.rollback()
                raise PGeoException(e.get_message(), e.get_status_code())
        except Exception, e:
            self.con.rollback()
            log.warn("Query error: use raise Exception? " + str(e))

    def __del__(self):
        self.close_connection()

    def __exit__(self):
        self.close_connection()

    def close_connection(self):
        if self.con is not None:
            self.con.close()
            log.info("Database '%s' connection closed. " % self.datasource['dbname'] )

    def get_connection_string(self, add_pg=True):
        db_connection_string = ""
        if add_pg is True:
            db_connection_string += "PG:"
        db_connection_string += "host=%s port='%s' dbname=%s user=%s password=%s" %(self.datasource['host'], self.datasource['port'],self.datasource['dbname'], self.datasource['username'], self.datasource['password'])
        return db_connection_string

    # blacklist methods not allowed
    def check_query(self, query):
        q = query.lower()
        if "insert" in q:
            return False
        if "update" in q:
            return False
        if "delete" in q:
            return False
        return True


# def get_connection_string(database, add_pg=True):
#     db_connection_string = ""
#     if add_pg is True:
#         db_connection_string += "PG:"
#         db_connection_string += "host=%s port='%s' dbname=%s user=%s password=%s" %(database['host'], database['port'],database['dbname'], database['username'], database['password'])
#     return db_connection_string


from setuptools import setup
from setuptools import find_packages

setup(
    name='PGeo',
    version='0.1.10',
    author='Simone Murzilli; Guido Barbaglia',
    author_email='geobrickspy@gmail.com',
    packages=find_packages(),
    install_requires=[
        'gdal',
        'pymongo',
        'psycopg2',
        'httplib2',
        'beautifulsoup4',
        'python-dateutil'
    ],
    url='http://pypi.python.org/pypi/PGeo/',
    license='LICENSE.txt',
    description='PGeo module.',
    long_description=open('README.md').read()
)



'''
thanks to https://github.com/boundlessgeo/gsconfig/tree/master/src/geoserver for the inspiration
'''

import urlparse
from tempfile import mkstemp
import urllib
from zipfile import ZipFile
import os

# TODO: remove the tmp directory to use a default filesystem path
# and clean it
def url(base, seg, query=None):
    """
    Create a URL from a list of path segments and an optional dict of query
    parameters.
    """
    def clean_segment(segment):
        """
        Cleans the segment and encodes to UTF-8 if the segment is unicode.
        """
        segment = segment.strip('/')
        if isinstance(segment, unicode):
            segment = segment.encode('utf-8')
        return segment

    seg = (urllib.quote(clean_segment(s)) for s in seg)
    if query is None or len(query) == 0:
        query_string = ''
    else:
        query_string = "?" + urllib.urlencode(query)
    path = '/'.join(seg) + query_string
    adjusted_base = base.rstrip('/') + '/'
    return urlparse.urljoin(adjusted_base, path)

def prepare_upload_bundle(name, data):
    """GeoServer's REST API uses ZIP archives as containers for file formats such
    as Shapefile and WorldImage which include several 'boxcar' files alongside
    the main data.  In such archives, GeoServer assumes that all of the relevant
    files will have the same base name and appropriate extensions, and live in
    the root of the ZIP archive.  This method produces a zip file that matches
    these expectations, based on a basename, and a dict of extensions to paths or
    file-like objects. The client code is responsible for deleting the zip
    archive when it's done."""
    fd, path = mkstemp()
    zip_file = ZipFile(path, 'w')
    print data.iteritems()
    for ext, stream in data.iteritems():
        fname = "%s.%s" % (name, ext)
        print fname
        if (isinstance(stream, basestring)):
            zip_file.write(stream, fname)
        else:
            zip_file.writestr(fname, stream.read())
    zip_file.close()
    os.close(fd)
    return path


from pgeo.db.postgresql.common import DBConnection


class DBStats:

    db = None
    schema=None
    datasource=None

    def __init__(self, datasource):
        if self.db is None:
            self.datasource = datasource
            self.db = DBConnection(self.datasource)
            self.schema = self.db.schema

    def insert_stats(self, table, data):
        try:
            return self.db.insert(table, data)
        except Exception, e:
            return False

    def query(self, query):
        return self.db.query(query)

    def get_connection_string(self, add_pg=True):
        db_connection_string = ""
        if add_pg is True:
            db_connection_string += "PG:"
            db_connection_string += "host=%s port='%s' dbname=%s user=%s password=%s" %(self.datasource['host'], self.datasource['port'],self.datasource['dbname'], self.datasource['username'], self.datasource['password'])
        return db_connection_string



from pgeo.gis import raster
from pgeo.utils import log
from pgeo.gis import geocoding
from os import path
import sys

log = log.logger("pgeo.gis.raster_test")



def processing_raster():
    config = {
        "descriptive_stats": {
            "force": True
        },
        "histogram": {
            "buckets": 256,
            "force": True
        }
    }
    raster_path = '/home/vortex/Desktop/LAYERS/TRMM/2014/07/output/trmm_07_2014.tif'
    query = "select * from spatial.gaul0_2013_4326 where adm0_code IN('68')"
    db_connection_string = "PG:host=localhost port=5432 dbname=pgeo user=fenix password=Qwaszx"

    raster_processed_path = raster.crop_by_vector_database(raster_path, query, db_connection_string)
    stats = raster.get_statistics(raster_processed_path, config)
    log.info(stats)



def location_values(lat, lon):
    #tiffs = glob.glob("/home/vortex/programs/layers/raster/TRMM/3B42RT/2014/03/original/*.tif")
    # tiffs = glob.glob("/home/vortex/programs/layers/raster/RASTER/Vegetation/NDVI/*.tif")
    #tiffs = glob.glob("/home/vortex/programs/layers/raster/TRMM/3B42RT/2014/03/original/geotiff/*.tif")
    tiffs = [
        "/home/vortex/Desktop/LAYERS/TRMM/2014/07/output/trmm_07_2014.tif"
    ]
    #return raster.location_values(tiffs, 41.005, 12.22)
    #return raster.location_values(tiffs, 1452160.76088, 5096412.999)
    return raster.location_values(tiffs, lat, lon)


# processing
processing_raster()

# lat lon
# l = geocoding.get_location("via bonn, aprilia")
# log.info(location_values(l.latitude, l.longitude))
# l = geocoding.get_location("via morrovalle 58, roma")
# log.info(location_values(l.latitude, l.longitude))







config = {

    # "raster" : {
    #     "name" : "MODISQ13",
    #
    #     # the stored UID in the GeoMetadata database (uses Geoserver Path (?) )
    #     "uid" : "trmm:3B42RT.2014010100.7.03hr",
    #
    #     # OR instead of the uid directly gives the path of the source layer without using the Default Path
    #     #"path" : "/hove/Desktop/GIS/layer.geojson",
    # },
    #
    # "vector" : {
    #
    #     # It's the name of the gaul spatial table
    #     "name" : "gaul0",
    #
    #     # Database
    #     "type" : "database",
    #     "options" : {
    #
    #         # used to query the db and retrieve the right codes
    #         "query_condition" : {
    #             "select": "adm0_code, adm0_name",
    #             "from": "{{SCHEMA}}.gaul0",
    #             "where": "adm0_code IN ('68', '69') GROUP BY adm0_code, adm0_name ",
    #             },
    #
    #         # used to subquery the db to get the geometry and process the raster
    #         "column_filter": "adm0_code",
    #
    #         # used to fill stats table (raster["name"].vector["name"])
    #         "stats_columns" : {
    #             "polygon_id" : "adm0_code",
    #             "label_en" : "adm0_name",
    #             }
    #     },
    #
    #     # TODO: GeoJson (Problem how to save the geojson fields? Just gives back the result without saving them)
    #     # "type" : "geojson",
    #     # "path" : "/hove/Desktop/GIS/layer.geojson",
    # },
    #
    # "stats" : {
    #     # default is false (return just the json with the statistics)
    #     "save_stats" : True,
    #
    #     # dynamically retrieved by the raster and vector names
    #     "table_name" : "raster.name_vector.name",
    #
    #     # table.sql contains Default CREATE TABLE
    #     "table_definition" : "$GEOMETADATA_DEFAULT_PATH/GAUL0/table.sql",
    #
    #     "table_insert": {
    #         # Table fiels to be used at runtime during the insertion
    #         # INSERT INTO table_name (polygon_id, label_en ..) VALUES (1, "Afghanistan, ..)
    #
    #         "polygon_id" :  "",
    #         "label_en" : "",
    #         "fromdate" : "",
    #         "todate"   : "",
    #         "dekad" : "",
    #         "hist" : "",
    #         "max" : "",
    #         "min" : "",
    #         "sd": ""
    #     },
    #
    #     # SQL containing INDEXES
    #     "table_indexes" : "$GEOMETADATA_DEFAULT_PATH/GAUL0/table_indexes.sql",
    #
    #     #  default option
    #     "delete_tmp_files" : True
    # }

}


import logging
from pgeo.config.settings import settings

level = settings["logging"]["level"]
format = settings["logging"]["format"]
datefmt = settings["logging"]["datefmt"]
logging.basicConfig(level=level,
                    format=format,
                    datefmt=datefmt)


def logger(loggerName=None):
    logger = logging.getLogger(loggerName)
    logger.setLevel(level)
    return logger





from os.path import join, isdir, islink
from os import listdir

def read_until(stream, separator, buffer_size=32*1024):
    datalist = []
    done = False
    while not done:
        chunk = stream.read(buffer_size)
        done = len(chunk) == 0
        while separator in chunk:
            i = chunk.find(separator)
            datalist.append(chunk[:i+1])
            yield ''.join(datalist)
            datalist = []
            if i < len(chunk) - 1:
                chunk = chunk[i + 1:len(chunk)]
            else:
                chunk = ''

        if chunk != '':
            datalist.append(chunk)

    if len(datalist) > 0:
        yield ''.join(datalist)

    return

def groupsgen(seq, size):
    it = iter(seq)
    while True:
        values = ()
        for n in xrange(size):
            try:
                values += (it.next(),)
            except StopIteration:
                yield values
                return
        yield values

from os.path import join, isdir, islink
from os import listdir

# modified os.walk() function from Python 2.4 standard library
# http://code.activestate.com/recipes/576933-modified-oswalk-which-return-current-directory-dep/
def walk2(top, topdown=True, onerror=None, deeplevel=0): # fix 0
    """Modified directory tree generator.

    For each directory in the directory tree rooted at top (including top
    itself, but excluding '.' and '..'), yields a 4-tuple

        dirpath, dirnames, filenames, deeplevel

    dirpath is a string, the path to the directory.  dirnames is a list of
    the names of the subdirectories in dirpath (excluding '.' and '..').
    filenames is a list of the names of the non-directory files in dirpath.
    Note that the names in the lists are just names, with no path components.
    To get a full path (which begins with top) to a file or directory in
    dirpath, do os.path.join(dirpath, name).

    ----------------------------------------------------------------------
    + deeplevel is 0-based deep level from top directory
    ----------------------------------------------------------------------
    ...

    """

    try:
        names = listdir(top)
    except Exception, err:
        if onerror is not None:
            onerror(err)
        return

    dirs, nondirs = [], []
    for name in names:
        if isdir(join(top, name)):
            dirs.append(name)
        else:
            nondirs.append(name)

    if topdown:
        yield top, dirs, nondirs, deeplevel # fix 1
    for name in dirs:
        path = join(top, name)
        if not islink(path):
            for x in walk2(path, topdown, onerror, deeplevel+1): # fix 2
                yield x
    if not topdown:
        yield top, dirs, nondirs, deeplevel # fix 3


import json
import os
import logging

settings = {

    # To be used by Flask: DEVELOPMENT ONLY
    "debug": True,

    # Flask host: DEVELOPMENT ONLY
    "host": "localhost",

    # Flask port: DEVELOPMENT ONLY
    "port": 5005,

    # Default folder root to store layers. Each data provider configuration file specifies the path AFTER this folder.
    "target_root": "/home/Desktop/GIS",

    # Each folder cntains one layer only. This is the default file name for such layers.
    "default_layer_name":  "layer.geotiff",

    # Logging configurations
    "logging": {
        "level": logging.INFO,
        "format": "%(asctime)s | %(levelname)-8s | %(name)-20s | Line: %(lineno)-5d | %(message)s",
        "datefmt": "%d-%m-%Y | %H:%M:%s"
    },

    "email": {
        "settings" : "/home/vortex/Desktop/LAYERS/email.json",
        "user" :  None,
        "password" : None
    },

    # Folders
    "folders": {
        "config": "config/",
        "tmp": "/home/vortex/Desktop/LAYERS/tmp",
        "data_providers": "data_providers/",
        "metadata": "metadata/",
        "stats": "stats/",
        "geoserver": "geoserver/",
        "metadata_templates": "metadata/templates/",
        # used on runtime statistics (for Published layers this is the Geoservers Cluster "datadir")
        "geoserver_datadir": "/home/vortex/programs/SERVERS/tomcat_geoservers/data/",
        #"geoserver_datadir": "/home/vortex/Desktop/LAYERS/GEOSERVER_TEST",

        "distribution": "/home/vortex/Desktop/LAYERS/DISTRIBUTION/"
    },

    # Databases
    "db": {
        "metadata": {
            "connection": "mongodb://localhost:27017/",
            "database": "metadata",
            "document": {
                "layer": "layer"
            }
        },

        # Spatial Database
        "spatial": {
            # default_db will search in the dbs["database"] as default option
            "dbname": "pgeo",
            "host": "localhost",
            "port": "5432",
            "username": "fenix",
            "password": "Qwaszx",
            "schema": "spatial"
        },

        "stats": {
            "dbname": "pgeo",
            "host": "localhost",
            "port": "5432",
            "username": "fenix",
            "password": "Qwaszx",
            "schema": "stats"
        }
    },

    # Geoserver
    "geoserver": {
        "geoserver_master": "http://localhost:9090/geoserver/rest",
        "geoserver_slaves": [],
        "username": "admin",
        "password": "geoserver",
        "default_workspace": "fenix",
        # this is used as default datasource to this is a reference to the spatial_db
        # da vedere!
        # this should be connected with the current spatial db
        "default_datastore": "pgeo"
    },

    # Stats
    "stats": {
        "db": {
            "spatial": "spatial",
            "stats": "stats"
        }
    },


    # Metadata
    "metadata": {

    }
}


def read_config_file_json(filename, folder=''):
    directory = os.path.dirname(os.path.dirname(__file__))
    filename = filename.lower()
    path = directory + '/' + settings['folders']['config'] + settings['folders'][folder]
    extension = '' if '.json' in filename else '.json'
    return json.loads(open(path + filename + extension).read())


def read_template(filename):
    try:
        directory = os.path.dirname(os.path.dirname(__file__))
        filename = filename.lower()
        path = os.path.join(directory, settings['folders']['config'], settings['folders']['metadata_templates'])
        extension = '' if '.json' in filename else '.json'
        return json.loads(open(path + filename + extension).read())
    except Exception, e:
        print e


def set_email_settings():
    print os.path.isfile(settings["email"]["settings"])
    if os.path.isfile(settings["email"]["settings"]):
        settings["email"] = json.loads(open(settings["email"]["settings"]).read())

set_email_settings()